# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  email: info@riskmap.net                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------
#  
#  Author: Enrico Sirola <enrico.sirola@statpro.com> 

"""drmaa errors"""

from ctypes import create_string_buffer
from somadrmaa.const import ERROR_STRING_BUFFER

class DrmaaException(Exception):
    """A common ancestor to all DRMAA Error classes."""
    pass

class AlreadyActiveSessionException(DrmaaException):
    pass
class AuthorizationException(DrmaaException):
    pass
class ConflictingAttributeValuesException(DrmaaException, AttributeError):
    pass
class DefaultContactStringException(DrmaaException):
    pass
class DeniedByDrmException(DrmaaException):
    pass
class DrmCommunicationException(DrmaaException):
    pass
class DrmsExitException(DrmaaException):
    pass
class DrmsInitException(DrmaaException):
    pass
class ExitTimeoutException(DrmaaException):
    pass
class HoldInconsistentStateException(DrmaaException):
    pass
class IllegalStateException(DrmaaException):
    pass
class InternalException(DrmaaException):
    pass
class InvalidAttributeFormatException(DrmaaException, AttributeError):
    pass
class InvalidContactStringException(DrmaaException):
    pass
class InvalidJobException(DrmaaException):
    pass
class InvalidJobTemplateException(DrmaaException):
    pass
class NoActiveSessionException(DrmaaException):
    pass
class NoDefaultContactStringSelectedException(DrmaaException):
    pass
class ReleaseInconsistentStateException(DrmaaException):
    pass
class ResumeInconsistentStateException(DrmaaException):
    pass
class SuspendInconsistentStateException(DrmaaException):
    pass
class TryLaterException(DrmaaException):
    pass
class UnsupportedAttributeException(DrmaaException, AttributeError):
    pass
class InvalidArgumentException(DrmaaException, AttributeError):
    pass
class InvalidAttributeValueException(DrmaaException, AttributeError):
    pass
class OutOfMemoryException(DrmaaException, MemoryError):
    pass

error_buffer = create_string_buffer(ERROR_STRING_BUFFER)

def error_check(code):
    if code == 0: return 
    else:
        try:
            raise _ERRORS[code-1]("code %s: %s" % (code, error_buffer.value))
        except IndexError:
            raise DrmaaException("code %s: %s" % (code, error_buffer.value))

# da vedere: NO_RUSAGE, NO_MORE_ELEMENTS
_ERRORS = [ InternalException,
            DrmCommunicationException,
            AuthorizationException,
            InvalidArgumentException,
            NoActiveSessionException,
            OutOfMemoryException,
            InvalidContactStringException,
            DefaultContactStringException,
            NoDefaultContactStringSelectedException,
            DrmsInitException,
            AlreadyActiveSessionException,
            DrmsExitException,
            InvalidAttributeFormatException,
            InvalidAttributeValueException,
            ConflictingAttributeValuesException,
            TryLaterException,
            DeniedByDrmException,
            InvalidJobException,
            ResumeInconsistentStateException,
            SuspendInconsistentStateException,
            HoldInconsistentStateException,
            ReleaseInconsistentStateException,
            ExitTimeoutException, 
            Exception, 
            StopIteration ]




'''
@author: Soizic Laguitton

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''


import copy
from soma_workflow.client import Workflow, Group, Job

def DetectFindLib(env_name, libname):
    '''Try to find libname using ctype
    
    '''
    import ctypes
    from ctypes.util import find_library
    from ctypes import CDLL
    from ctypes import cdll
    import os
    import glob
    
    libpath=None
    IS_LIB_FOUND=True
    _lib = None

    if env_name in os.environ:
        libpath = os.environ[env_name]
    else:
        libpath = find_library(libname)
    
    libsoname="lib"+libname+".so"
    if "LD_LIBRARY_PATH" in os.environ:
        ld_lib_paths=os.environ["LD_LIBRARY_PATH"]
        ld_lib_paths=ld_lib_paths.split(os.pathsep)
        for ld_lib_path in ld_lib_paths:
            files=glob.glob(os.path.join(ld_lib_path,libsoname))
            if(len(files)>=1):
                libpath=files[0]
                break


    if libpath is None:
#         try:
#             soname="lib"+libname+".so"
#             _lib=cdll.LoadLibrary(soname,mode=ctypes.RTLD_GLOBAL)
#         except OSError, e:
#             IS_LIB_FOUND=False
#         except:
#             IS_LIB_FOUND=False
        IS_LIB_FOUND=False
    else :
        _lib = CDLL(libpath, mode=ctypes.RTLD_GLOBAL)
        
    return (IS_LIB_FOUND,_lib)


def process_group(group, to_remove, name):
  new_group = []
  for element in group:
    if isinstance(element, Job):
      if element in to_remove:
        if not to_remove[element] in new_group:
          new_group.append(to_remove[element])
      else:
        new_group.append(element)
    else:
      
      new_group.append(process_group(element.elements, to_remove, element.name))  
  return Group(new_group, name)


class SerialJob(object):

  _job_sequence = None
  _input_dep = None
  _output_dep = None
  _working_directory = None
  _group = None

  def __init__(self):
    self._job_sequence = []
    self._input_dep = []
    self._output_dep = []
    self._working_directory = None
    self._group = None

  def job_sequence(self):
    return self._job_sequence

  def input_dep(self):
    return self._input_dep

  def output_dep(self):
    return self._output_dep

  def working_directory(self):
    return self._working_directory

  def group(self):
    return self._group

  def is_valid(self):
    return len(self._job_sequence) > 1
    
  def add_job(self, job, group, input_dep, output_dep):
    assert self.is_consistent(job, group)

    if not self._job_sequence:
      self._working_directory = job.working_directory
      self._group = group
      self._input_dep = input_dep
    
    self._job_sequence.append(job)
    self._output_dep = output_dep

  def is_consistent(self, job, group):
    if not self._job_sequence:
      return True

    is_consistent = job.working_directory == self._working_directory and \
                    group == self._group and \
                    job.stdin == None and \
                    job.stdout_file == None and \
                    job.stderr_file == None and \
                    job.parallel_job_info == None
    return is_consistent
           


def explore(root_job, 
            current_serial_job,
            serial_jobs, 
            explored, 
            workflow):

  if root_job in explored:
    return 

  input_dep = []
  output_dep = []
  for dep in workflow.dependencies:
    if dep[0] == root_job:
      output_dep.append(dep[1])
    elif dep[1] == root_job:
      input_dep.append(dep[0])

  group = None
  if root_job in workflow.root_group:
    group = workflow.root_group
  else:
    for gp in workflow.groups:
      if root_job in gp.elements:
        group = gp 
        break

  explored.add(root_job)

  if len(input_dep) == 1:
    if current_serial_job.is_consistent(root_job, group):
      current_serial_job.add_job(root_job, group, input_dep, output_dep)
    else:
      if current_serial_job.is_valid():
        serial_jobs.append(current_serial_job)
      current_serial_job = SerialJob()
      current_serial_job.add_job(root_job, group, input_dep, output_dep)

    if len(output_dep) > 1:
      if current_serial_job.is_valid():
        serial_jobs.append(current_serial_job)
      for job in output_dep:
        current_serial_job = SerialJob()
        explore(job, 
                current_serial_job, 
                serial_jobs, 
                explored, 
                workflow)
    elif len(output_dep) == 0:
      if current_serial_job.is_valid():
        serial_jobs.append(current_serial_job)
    elif len(output_dep) == 1:
      explore(output_dep[0], 
              current_serial_job, 
              serial_jobs, 
              explored, 
              workflow)

  elif len(input_dep) == 0:
    if len(output_dep) == 1:
      current_serial_job = SerialJob()
      current_serial_job.add_job(root_job, group, input_dep, output_dep)
      explore(output_dep[0],
              current_serial_job,
              serial_jobs,
              explored,
              workflow)
    elif len(output_dep) > 1:
      for job in output_dep:
        current_serial_job = SerialJob()
        explore(job, 
                current_serial_job, 
                serial_jobs, 
                explored, 
                workflow)

  elif len(input_dep) > 1:
    if current_serial_job.is_valid():
      serial_jobs.append(current_serial_job)
    current_serial_job = SerialJob()
    if len(output_dep) == 1:
      current_serial_job.add_job(root_job, group, input_dep, output_dep)  
      explore(output_dep[0],
              current_serial_job,
              serial_jobs,
              explored,
              workflow)
      
    elif len(output_dep) > 1:
      for job in output_dep:
        current_serial_job = SerialJob()
        explore(job, 
                current_serial_job, 
                serial_jobs, 
                explored, 
                workflow)


def checkFiles(files, filesModels, tolerance = 0):
  index = 0
  for file in files:
    t = tolerance
    (identical, msg) = identicalFiles(file, filesModels[index])
    if not identical: 
      if t <= 0: 
        return (identical, msg)
      else: 
        t = t -1
        print "\n checkFiles: "+ msg
    index = index +1
  return (True, None)


def identicalFiles(filepath1, filepath2):
  file1 = open(filepath1)
  file2 = open(filepath2)
  lineNb = 1
  line1 = file1.readline()
  line2 = file2.readline()
  identical = line1 == line2
  
  while identical and line1 :
    line1 = file1.readline()
    line2 = file2.readline()
    lineNb = lineNb + 1
    identical = line1 == line2
  
  if identical: identical = line1 == line2
  if not identical:
    return (False, "%s and %s are different. line %d: \n file1: %s file2:%s" %(filepath1, filepath2, lineNb, line1, line2))
  else:
    return (True, None)



'''
@author: Soizic Laguitton

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''

class SomaWorkflowError(Exception):
  '''
  Wrong usage of Soma-workflow.
  '''
  pass


class ConfigurationError(Exception): 
  '''
  Raised when the configuration is not correct or when configuration
  items are missing.
  '''
  pass 

class ConnectionError(Exception):   
  '''
  Raised when the connection could not be set up with the computing 
  resource and/or the database server.
  '''
  pass

class DRMError(Exception):          
  '''
  DRMAA session error or DRMS error.
  '''
  pass

class JobError(Exception):           
  '''
  Raised when an incorrect Job is submitted to soma-workflow.
  '''
  pass

class WorkflowError(Exception):      
  '''
  Raised when an incorrect Workflow is submitted to soma-workflow.
  '''
  pass

class TransferError(Exception):
  '''
  Raised when an incorrect Transfer is submitted to soma_workflow, or when an error occurs during a file transfer.
  '''
  pass

class UnknownObjectError(Exception):
  '''
  Raised when an operation is attempted on Jobs, Workflows or Transfers which
  do not exist or do not belong to the user.
  '''
  pass


class EngineError(Exception):
  '''
  Raised if an error occurs while starting the workflow engine. 
  '''
  pass


class DatabaseError(Exception):
  '''
  Raised if an error occurs while reading or writing to the database.
  '''
  pass


class SerializationError(Exception):
  pass


class NoDrmaaLibError(Exception):
  '''
  Raised if no drmaa libary is found
  '''
  pass




from __future__ import with_statement 

'''
@author: Soizic Laguitton
@organization: U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''


#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import unittest
import os
import getpass
import sys

from soma_workflow.client import Job, SharedResourcePath, FileTransfer, Group, Workflow, WorkflowController, Helper
from soma_workflow.configuration import Configuration
from soma_workflow.errors import ConfigurationError, UnknownObjectError
from soma_workflow.utils import checkFiles, identicalFiles
import soma_workflow.constants as constants


#-----------------------------------------------------------------------------
# Classes and functions
#-----------------------------------------------------------------------------


class WorkflowExamples(object):
  
  @staticmethod
  def get_workflow_example_list():
    return ["simple", "multiple", "with exception 1", "with exception 2", "command check test", "special transfers", "hundred of jobs", "ten jobs", "fake pipelineT1", "serial", "hundred with dependencies", "thousands", "thousands with dependencies", "native specification for PBS", "wrong native specification for PBS"]
  
  def __init__(self, with_transfers, with_shared_resource_path = False):
    '''
    @type with_transfers: boolean
    @type with_shared_resource_path: boolean

    if with_transfer and not with_shared_resource_path:
      The input and ouput files are temporary files on the computing resource
      and these files can be transfered from and to the computing resource
      using soma workflow API

    if with_shared_resource_path and not with_transfers:
      The files are read and written on the computing resource.

    if with_shared_resource_path and with_transfer:
      The files are read from data located on the computing resource but the output
      will be written in temporary files and transfered to the computing resource.
      
    '''
    self.examples_dir = os.environ.get("SOMA_WORKFLOW_EXAMPLES")
    self.output_dir = os.environ.get("SOMA_WORKFLOW_EXAMPLES_OUT")

    if not self.examples_dir or not self.output_dir:
      raise ConfigurationError("The environment variables SOMA_WORKFLOW_EXAMPLES "  
                               "and SOMA_WORKFLOW_EXAMPLES_OUT must be set.")
    
    if not os.path.isdir(self.output_dir):
      os.mkdir(self.output_dir)
       
    self.with_transfers = with_transfers
    self.with_shared_resource_path = with_shared_resource_path
    
    # local path
    
    complete_path = os.path.join(self.examples_dir, "complete")
    self.lo_in_dir = self.examples_dir
    self.lo_img_file = os.path.join(self.examples_dir, "special_transfers/example.img")
    self.lo_file0   = os.path.join(complete_path, "file0") 
    self.lo_script1 = os.path.join(complete_path, "job1.py")
    self.lo_stdin1  = os.path.join(complete_path, "stdin1")
    self.lo_script2 = os.path.join(complete_path, "job2.py")
    self.lo_stdin2  = os.path.join(complete_path, "stdin2")
    self.lo_script3 = os.path.join(complete_path, "job3.py")
    self.lo_stdin3  = os.path.join(complete_path, "stdin3")
    self.lo_script4 = os.path.join(complete_path, "job4.py")
    self.lo_stdin4  = os.path.join(complete_path, "stdin4")
    self.lo_exceptionJobScript = os.path.join(self.examples_dir, 
                                              "simple/exceptionJob.py")
    self.lo_cmd_check_script = os.path.join(self.examples_dir, 
                                            "command/argument_check.py")
    self.lo_sleep_script = os.path.join(self.examples_dir, "simple/sleep_job.py")
                                            
    self.lo_dir_contents_script = os.path.join(self.examples_dir, 
                                            "special_transfers/dir_contents.py")
    self.lo_mff_script = os.path.join(self.examples_dir, "special_transfers/multiple_file_format.py")
    
    self.lo_file11 = os.path.join(self.output_dir, "file11")
    self.lo_file12 = os.path.join(self.output_dir, "file12")
    self.lo_file2 = os.path.join(self.output_dir, "file2")
    self.lo_file3 = os.path.join(self.output_dir, "file3")
    self.lo_file4 = os.path.join(self.output_dir, "file4")
    self.lo_out_dir = os.path.join(self.output_dir, "transfered_dir")
    self.lo_img_out_file = os.path.join(self.output_dir, "example.img")

    self.lo_stdout1  = os.path.join(complete_path, "outputModels/stdoutjob1")
    self.lo_stdout2  = os.path.join(complete_path, "outputModels/stdoutjob2")
    self.lo_stdout3  = os.path.join(complete_path, "outputModels/stdoutjob3")
    self.lo_stdout4  = os.path.join(complete_path, "outputModels/stdoutjob4")
    
    self.lo_stdout1_exception_model = os.path.join(self.examples_dir, "simple/outputModels/stdout_exception_job")

    self.lo_stderr1  = os.path.join(complete_path, "outputModels/stderrjob1")
    self.lo_stderr2  = os.path.join(complete_path, "outputModels/stderrjob2")
    self.lo_stderr3  = os.path.join(complete_path, "outputModels/stderrjob3")
    self.lo_stderr4  = os.path.join(complete_path, "outputModels/stderrjob4")

    self.lo_out_model_file11		= os.path.join(complete_path, "outputModels/file11")
    self.lo_out_model_file12		= os.path.join(complete_path, "outputModels/file12")
    self.lo_out_model_file2			= os.path.join(complete_path, "outputModels/file2")
    self.lo_out_model_file3			= os.path.join(complete_path, "outputModels/file3")
    self.lo_out_model_file4			= os.path.join(complete_path, "outputModels/file4")
  
    # Shared resource path
    self.sh_in_dir = SharedResourcePath("", "example", "job_dir", 168)
    self.sh_img_file = SharedResourcePath("special_transfers/example.img", "example", "job_dir", 168)
    self.sh_file0   = SharedResourcePath("complete/file0", "example", "job_dir", 168)
    self.sh_script1 = SharedResourcePath("complete/job1.py", "example", "job_dir", 168)
    self.sh_stdin1  = SharedResourcePath("complete/stdin1", "example", "job_dir", 168)
    self.sh_script2 = SharedResourcePath("complete/job2.py", "example", "job_dir", 168)
    self.sh_stdin2  = SharedResourcePath("complete/stdin2", "example", "job_dir", 168)
    self.sh_script3 = SharedResourcePath("complete/job3.py", "example", "job_dir", 168)
    self.sh_stdin3  = SharedResourcePath("complete/stdin3", "example", "job_dir", 168)
    self.sh_script4 = SharedResourcePath("complete/job4.py", "example", "job_dir", 168)
    self.sh_stdin4  = SharedResourcePath("complete/stdin4", "example", "job_dir", 168)
    self.sh_exceptionJobScript = SharedResourcePath("simple/exceptionJob.py", 
                                                    "example", 
                                                    "job_dir", 168)
    self.sh_cmd_check_script = SharedResourcePath("command/argument_check.py",
                                                  "example",
                                                  "job_dir", 168)
    self.sh_sleep_script = SharedResourcePath("simple/sleep_job.py", 
                                              "example", 
                                              "job_dir", 168)
    self.sh_dir_contents_script = SharedResourcePath("special_transfers/dir_contents.py",
                                                     "example",
                                                     "job_dir", 168)
    self.sh_mff_script = SharedResourcePath("special_transfers/multiple_file_format.py", 
                       "example", "job_dir", 168)
   
    
    self.sh_file11 = SharedResourcePath("file11", "example", "output_dir", 168)
    self.sh_file12 = SharedResourcePath("file12", "example", "output_dir",168)
    self.sh_file2 = SharedResourcePath("file2", "example", "output_dir",168)
    self.sh_file3 = SharedResourcePath("file3", "example", "output_dir",168)
    self.sh_file4 = SharedResourcePath("file4", "example", "output_dir",168)
    self.sh_out_dir = SharedResourcePath("transfered_dir", "example", "output_dir", 168) 
    self.sh_img_out_file = SharedResourcePath("example.img", "example", "output_dir", 168)

    # Transfers
    
    complete_path = os.path.join(self.examples_dir, "complete")
    self.tr_in_dir = FileTransfer(True, self.examples_dir, 168, "in_dir")
    self.tr_img_file = FileTransfer(True, 
                                    os.path.join(self.examples_dir, 
                                                 "special_transfers/dir_contents.py"), 
                                    128, 
                                    "img_file", 
                                    [ os.path.join(self.examples_dir, "special_transfers/example.img"),
                                      os.path.join(self.examples_dir, "special_transfers/example.hdr")])
    self.tr_file0   = FileTransfer(True,os.path.join(complete_path, "file0"), 168, "file0")
    self.tr_script1 = FileTransfer(True,os.path.join(complete_path, "job1.py"), 168, "job1_py")
    self.tr_stdin1  = FileTransfer(True,os.path.join(complete_path, "stdin1"), 168, "stdin1")
    self.tr_script2 = FileTransfer(True,os.path.join(complete_path, "job2.py"), 168, "job2_py")
    self.tr_stdin2  = FileTransfer(True,os.path.join(complete_path, "stdin2"), 168, "stdin2")
    self.tr_script3 = FileTransfer(True,os.path.join(complete_path, "job3.py"), 168, "job3_py")
    self.tr_stdin3  = FileTransfer(True,os.path.join(complete_path, "stdin3"), 168, "stdin3")
    self.tr_script4 = FileTransfer(True,os.path.join(complete_path, "job4.py"), 168, "job4_py")
    self.tr_stdin4  = FileTransfer(True,os.path.join(complete_path, "stdin4"), 168, "stdin4")
    self.tr_exceptionJobScript = FileTransfer(True,os.path.join(self.examples_dir, 
                                                          "simple/exceptionJob.py"), 
                                                          168, "exception_job")
    self.tr_cmd_check_script = FileTransfer(True,os.path.join(self.examples_dir, 
                                                        "command/argument_check.py"),
                                                        168, "cmd_check")
    self.tr_sleep_script = FileTransfer(True,os.path.join(self.examples_dir, 
                                                    "simple/sleep_job.py"), 
                                                     168, "sleep_job")
    self.tr_dir_contents_script = FileTransfer(True,os.path.join(self.examples_dir, 
                                                        "special_transfers/dir_contents.py"),
                                                        168, "dir_contents")
    self.tr_mff_script = FileTransfer(True, os.path.join(self.examples_dir, "special_transfers/multiple_file_format.py"), 168, "mdd_script")
    
    self.tr_file11 = FileTransfer(False,os.path.join(self.output_dir, "file11"), 168, "file11")
    self.tr_file12 = FileTransfer(False,os.path.join(self.output_dir, "file12"), 168, "file12")
    self.tr_file2 = FileTransfer(False,os.path.join(self.output_dir, "file2"), 168, "file2")
    self.tr_file3 = FileTransfer(False,os.path.join(self.output_dir, "file3"), 168, "file3")
    self.tr_file4 = FileTransfer(False,os.path.join(self.output_dir, "file4"), 168, "file4")
    self.tr_out_dir = FileTransfer(False, os.path.join(self.output_dir, "transfered_dir"), 168, "out_dir")
    self.tr_img_out_file = FileTransfer(False, 
                                        os.path.join(self.output_dir, "example.img"), 
                                        168, 
                                        "img_out",
                                        [os.path.join(self.output_dir, "example.img"),
                                         os.path.join(self.output_dir, "example.hdr")])
  
      
  def get_workflow_example(self, example_index):
    workflow = None
    if example_index == 0:
      workflow = self.simple_example()
    elif example_index == 1:
      workflow = self.multiple_simple_example()
    elif example_index == 2:
      workflow = self.simple_example_with_exception1()
    elif example_index == 3:
      workflow = self.simple_example_with_exception2()
    elif example_index == 4:
      workflow = self.special_command()
    elif example_index == 5:
      workflow = self.special_transfer()
    elif example_index == 6:
      workflow = self.n_jobs(100)
    elif example_index == 7:
      workflow = self.n_jobs(10)
    elif example_index == 8:
      workflow = self.fake_pipelineT1()
    elif example_index == 9:
      workflow = self.serial_jobs()
    elif example_index == 10:
      workflow = self.n_jobs_with_dependencies(500)
    elif example_index == 11:
      workflow = self.n_jobs(12000)
    elif example_index == 12:
      workflow = self.n_jobs_with_dependencies(2000)
    elif example_index == 13:
      workflow = self.native_spec_pbs()
    elif example_index == 14:
      workflow = self.wrong_native_spec_pbs()
    return workflow

  def job1(self, option=None):
    time_to_wait=2
    job_name="job1"
    if self.with_transfers and not self.with_shared_resource_path: 
      job1 = Job(["python", self.tr_script1, self.tr_file0,  self.tr_file11, self.tr_file12, repr(time_to_wait)], 
                  [self.tr_file0, self.tr_script1, self.tr_stdin1], 
                  [self.tr_file11, self.tr_file12], 
                  self.tr_stdin1, False, 168, job_name, native_specification=option)
    elif self.with_transfers and self.with_shared_resource_path:
      job1 = Job( ["python", self.sh_script1, self.sh_file0,  self.tr_file11, self.tr_file12, repr(time_to_wait)], 
                  [], 
                  [self.tr_file11, self.tr_file12], 
                  self.sh_stdin1, False, 168, job_name, native_specification=option)
    elif not self.with_transfers and self.with_shared_resource_path:
      job1 = Job( ["python", self.sh_script1, self.sh_file0,  self.sh_file11, self.sh_file12, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.sh_stdin1, False, 168, job_name, native_specification=option)
    else:
      job1 = Job( ["python", self.lo_script1, self.lo_file0,  self.lo_file11, self.lo_file12, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.lo_stdin1,  False, 168, job_name, native_specification=option)
    return job1
    
  def job2(self):
    time_to_wait=2
    job_name="job2"
    if self.with_transfers and not self.with_shared_resource_path:
      job2 = Job( ["python", self.tr_script2, self.tr_file11,  self.tr_file0, self.tr_file2, repr(time_to_wait)], 
                  [self.tr_file0, self.tr_file11, self.tr_script2, self.tr_stdin2], 
                  [self.tr_file2], 
                  self.tr_stdin2, False, 168, job_name)
    elif self.with_transfers and self.with_shared_resource_path:
      job2 = Job( ["python", self.sh_script2, self.sh_file11,  self.sh_file0, self.tr_file2, repr(time_to_wait)], 
                  [], 
                  [self.tr_file2], 
                  self.sh_stdin2, False, 168, job_name)
    elif not self.with_transfers and self.with_shared_resource_path:
      job2 = Job( ["python", self.sh_script2, self.sh_file11,  self.sh_file0, self.sh_file2, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.sh_stdin2, False, 168, job_name)
    else:
      job2 = Job( ["python", self.lo_script2, self.lo_file11,  self.lo_file0, self.lo_file2, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.lo_stdin2, False, 168, job_name)
    return job2
    
  def job3(self):
    time_to_wait=2
    job_name="job3"
    if self.with_transfers and not self.with_shared_resource_path:
      job3 = Job( ["python", self.tr_script3, self.tr_file12,  self.tr_file3, repr(time_to_wait)], 
                  [self.tr_file12, self.tr_script3, self.tr_stdin3], 
                  [self.tr_file3], 
                  self.tr_stdin3, False, 168, job_name)
    elif self.with_transfers and self.with_shared_resource_path:
      job3 = Job( ["python", self.sh_script3, self.sh_file12,  self.tr_file3, repr(time_to_wait)], 
                  [], 
                  [self.tr_file3], 
                  self.sh_stdin3, False, 168, job_name)
    elif not self.with_transfers and self.with_shared_resource_path:
      job3 = Job( ["python", self.sh_script3, self.sh_file12,  self.sh_file3, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.sh_stdin3, False, 168, job_name)
    else:
      job3 = Job( ["python", self.lo_script3, self.lo_file12,  self.lo_file3, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.lo_stdin3,  False, 168, job_name)
    return job3
    
  def job4(self):
    time_to_wait=10
    job_name="job4"
    if self.with_transfers and not self.with_shared_resource_path:
      job4 = Job( ["python", self.tr_script4, self.tr_file2,  self.tr_file3, self.tr_file4, repr(time_to_wait)], 
                  [self.tr_file2, self.tr_file3, self.tr_script4, self.tr_stdin4], 
                  [self.tr_file4], 
                  self.tr_stdin4, False, 168, job_name)
    elif self.with_transfers and self.with_shared_resource_path:
      job4 = Job( ["python", self.sh_script4, self.sh_file2,  self.sh_file3, self.tr_file4, repr(time_to_wait)], 
                  [], 
                  [self.tr_file4], 
                  self.sh_stdin4, False, 168, job_name)
    elif not self.with_transfers and self.with_shared_resource_path:
      job4 = Job( ["python", self.sh_script4, self.sh_file2,  self.sh_file3, self.sh_file4, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.sh_stdin4, False, 168, job_name)
    else:
      job4 = Job( ["python", self.lo_script4, self.lo_file2,  self.lo_file3, self.lo_file4, repr(time_to_wait)], 
                  None, 
                  None, 
                  self.lo_stdin4, False, 168, job_name)
    return job4
    
  def job_test_command_1(self):
    if self.with_transfers:
      test_command = Job( ["python", 
                           self.tr_cmd_check_script,
                           [self.tr_script1, self.tr_script2, self.tr_script3], 
                           "[13.5, 14.5, 15.0]",
                           '[13.5, 14.5, 15.0]',
                           "['un', 'deux', 'trois']",
                           '["un", "deux", "trois"]'],
                          [self.tr_cmd_check_script, self.tr_script1, self.tr_script2, self.tr_script3],
                          [],
                          None, False, 168, "test_command_1")
    elif self.with_shared_resource_path:
      test_command = Job( ["python", self.sh_cmd_check_script,
                           [self.sh_script1, self.sh_script2, self.sh_script3],
                           "[13.5, 14.5, 15.0]",
                           '[13.5, 14.5, 15.0]',
                           "['un', 'deux', 'trois']",
                           '["un", "deux", "trois"]'],
                          None,
                          None,
                          None, False, 168, "test_command_1")
    else:
      test_command = Job( ["python", self.lo_cmd_check_script, 
                           [self.lo_script1, self.lo_script2, self.lo_script3],
                           "[13.5, 14.5, 15.0]",
                           '[13.5, 14.5, 15.0]',
                           "['un', 'deux', 'trois']",
                           '["un", "deux", "trois"]'],
                          None,
                          None,
                          None, False, 168, "test_command_1")
    
    return test_command
    
    
  def job_test_dir_contents(self):
    if self.with_transfers:
      job = Job(["python", 
                self.tr_dir_contents_script,
                self.tr_in_dir,
                self.tr_out_dir],
                [self.tr_dir_contents_script, self.tr_in_dir],
                [self.tr_out_dir],
                None, False, 168, "dir_contents")
    elif self.with_shared_resource_path:
      job = Job(["python", 
                self.sh_dir_contents_script, 
                self.sh_in_dir,
                self.sh_out_dir],
                None,
                None,
                None, False, 168, "dir_contents")
    else:
      job = Job(["python", 
                self.lo_dir_contents_script, 
                self.lo_in_dir,
                self.lo_out_dir],
                None,
                None,
                None, False, 168, "dir_contents")
    
    return job

  def job_test_multi_file_format(self):
    if self.with_transfers:
      job = Job( ["python", 
                  self.tr_mff_script,
                  (self.tr_img_file, "example.img"),
                  (self.tr_img_out_file, "example.img")],
                [self.tr_mff_script, self.tr_img_file],
                [self.tr_img_out_file],
                None, False, 168, "multi file format test")
    elif self.with_shared_resource_path:
      job = Job( ["python", 
                  self.sh_mff_script, 
                  self.sh_img_file,
                  self.sh_img_out_file],
                None,
                None,
                None, False, 168, "multi file format test")
    else:
      job = Job( ["python", 
                  self.lo_mff_script, 
                  self.lo_img_file,
                  self.lo_img_out_file],
                None,
                None,
                None, False, 168, "multi file format test")
    
    return job
    
    
  def job_sleep(self, period):
    if self.with_transfers:
      complete_path = os.path.join(self.examples_dir, "complete")
      transfer   = FileTransfer(True,os.path.join(complete_path, "file0"), 168, "file0")
      job = Job( ["python", self.tr_sleep_script, repr(period)],
                 [self.tr_sleep_script, transfer],
                 [],
                 None, False, 168, "sleep " + repr(period) + " s")
    elif self.with_shared_resource_path:
      job = Job( ["python", self.sh_sleep_script, repr(period)],
                 None,
                 None,
                 None, False, 168, "sleep " + repr(period) + " s")
    else:
      job = Job( ["python", self.lo_sleep_script, repr(period)],
                 None,
                 None,
                 None, False, 168, "sleep " + repr(period) + " s")
    
    return job
    
  def special_transfer(self):
     # jobs
    test_dir_contents = self.job_test_dir_contents()
    test_multi_file_format = self.job_test_multi_file_format()
        
    # building the workflow
    jobs = [test_dir_contents, test_multi_file_format]
    dependencies = []
       
    workflow = Workflow(jobs, dependencies)
    
    return workflow
      

  def special_command(self):
    
    # jobs
    test_command_job = self.job_test_command_1()
        
    # building the workflow
    jobs = [test_command_job]
    
    dependencies = []
     
    workflow = Workflow(jobs, dependencies)
    
    return workflow
      
  def simple_example(self):
    
    # jobs
    job1 = self.job1()
    job2 = self.job2()
    job3 = self.job3()
    job4 = self.job4()
    
    #building the workflow
    jobs = [job1, job2, job3, job4]
  
    dependencies = [(job1, job2), 
                    (job1, job3),
                    (job2, job4), 
                    (job3, job4)]
  
    group_1 = Group(name='group_1', elements=[job2, job3])
    group_2 = Group(name='group_2', elements=[job1, group_1])
    
    workflow = Workflow(jobs, dependencies, root_group=[group_2, job4])
    
    return workflow

  def wrong_native_spec_pbs(self):
     # jobs
    job1 = self.job1(option="-l walltime=5:00:00, pmem=16gb")
    job2 = self.job1(option="-l walltime=5:00:0")
    job3 = self.job1()
    
    #building the workflow
    jobs = [job1, job2, job3]
    
    workflow = Workflow(jobs, dependencies=[], name="jobs with wrong native spec for pbs")
    
    return workflow


  def native_spec_pbs(self):
     # jobs
    job1 = self.job1(option="-l walltime=5:00:00,pmem=16gb")
    job2 = self.job1(option="-l walltime=5:00:0")
    job3 = self.job1()
    
    #building the workflow
    jobs = [job1, job2, job3]
    
    workflow = Workflow(jobs, dependencies=[], name="jobs with native spec for pbs")
    
    return workflow

 
  
  def simple_example_with_exception1(self):
                                                          
    # jobs
    if self.with_transfers and not self.with_shared_resource_path:
      job1 = Job( ["python", self.tr_exceptionJobScript], 
                  [self.tr_exceptionJobScript, self.tr_file0, self.tr_script1, self.tr_stdin1], 
                  [self.tr_file11, self.tr_file12], 
                  self.tr_stdin1, False, 168, "job1 with exception")
    elif self.with_transfers and self.with_shared_resource_path:
      job1 = Job( ["python", self.sh_exceptionJobScript], 
                  [], 
                  [self.tr_file11, self.tr_file12], 
                  self.sh_stdin1, False, 168, "job1 with exception")
    elif not self.with_transfers and self.with_shared_resource_path:
      job1 = Job( ["python", self.sh_exceptionJobScript], 
                  None, 
                  None, 
                  self.sh_stdin1, False, 168, "job1 with exception")
    else:
      job1 = Job( ["python", self.lo_exceptionJobScript], 
                  None, 
                  None,  
                  self.lo_stdin1, False, 168, "job1 with exception")                   
    
    job2 = self.job2()
    job3 = self.job3()
    job4 = self.job4()
          
    jobs = [job1, job2, job3, job4]
  

    dependencies = [(job1, job2), 
                    (job1, job3),
                    (job2, job4), 
                    (job3, job4)]
  
    group_1 = Group(name='group_1', elements=[job2, job3])
    group_2 = Group(name='group_2', elements=[job1, group_1])
   
    workflow = Workflow(jobs, dependencies, root_group=[group_2, job4])
    
    return workflow
  
  
  def simple_example_with_exception2(self):
   
    # jobs
    job1 = self.job1()
    job2 = self.job2()
    job4 = self.job4()
    
    if self.with_transfers and not self.with_shared_resource_path:
      job3 = Job( ["python", self.tr_exceptionJobScript],
                  [self.tr_exceptionJobScript, self.tr_file12, self.tr_script3, self.tr_stdin3],
                  [self.tr_file3],
                  self.tr_stdin3, False, 168, "job3 with exception")
    elif self.with_transfers and not self.with_shared_resource_path:
      job3 = Job( ["python", self.sh_exceptionJobScript],
                  [],
                  [self.tr_file3],
                  self.sh_stdin3, False, 168, "job3 with exception")
    elif self.with_transfers and not self.with_shared_resource_path:
      job3 = Job( ["python", self.sh_exceptionJobScript],
                  None,
                  None,
                  self.sh_stdin3, False, 168, "job3 with exception")
    else:
      job3 = Job( ["python", self.lo_exceptionJobScript], 
                  None, 
                  None, 
                  self.lo_stdin3, False, 168, "job3 with exception")
      
      
           
    jobs = [job1, job2, job3, job4]

    dependencies = [(job1, job2), 
                    (job1, job3),
                    (job2, job4), 
                    (job3, job4)]
  
    group_1 = Group(name='group_1', elements=[job2, job3])
    group_2 = Group(name='group_2', elements=[job1, group_1])
     
    workflow = Workflow(jobs, dependencies, root_group=[group_2, job4])
    
    return workflow
  
  
  def multiple_simple_example(self):
    workflow1 = self.simple_example()
    workflow2 = self.simple_example_with_exception1()
    workflow3 = self.simple_example_with_exception2()
    
    jobs = workflow1.jobs
    jobs.extend(workflow2.jobs)
    jobs.extend(workflow3.jobs)
    
    dependencies = workflow1.dependencies
    dependencies.extend(workflow2.dependencies)
    dependencies.extend(workflow3.dependencies)
    
    group1 = Group(name="simple example", elements=workflow1.root_group)
    group2 = Group(name="simple with exception in Job1",          
                   elements=workflow2.root_group)
    group3 = Group(name="simple with exception in Job3",
                   elements=workflow3.root_group)
     
    workflow = Workflow(jobs, dependencies, root_group=[group1, group2, group3])
    return workflow
  
  
  def n_jobs(self, nb=300, time=60):
      
    jobs = []
    for i in range(0,nb):
      job = self.job_sleep(time)
      jobs.append(job)
     
    dependencies = []
    
    workflow = Workflow(jobs, dependencies)
    
    return workflow
  

  def n_jobs_with_dependencies(self, nb=500, time=60):
    
    dependencies = []
    jobs = []
    intermed_job1 = self.job_sleep(2)
    jobs.append(intermed_job1)
    intermed_job2 = self.job_sleep(2)
    jobs.append(intermed_job2)

    elem_group1 = []
    for i in range(0, nb):
      job = self.job_sleep(time)
      jobs.append(job)
      elem_group1.append(job)
      dependencies.append((job,intermed_job1))
    group1 = Group(name="Group 1", elements=elem_group1)

    elem_group2 = []
    for i in range(0, nb):
      job = self.job_sleep(time)
      jobs.append(job)
      elem_group2.append(job)
      dependencies.append((intermed_job1, job))
      dependencies.append((job, intermed_job2))
    group2 = Group(name="Group 2", elements=elem_group2)

    elem_group3 = []
    for i in range(0, nb):
      job = self.job_sleep(time)
      jobs.append(job)
      elem_group3.append(job)
      dependencies.append((intermed_job2, job))
    group3 = Group(name="Group 3", elements=elem_group3)

    root_group = [group1, intermed_job1, group2, intermed_job2, group3]
    workflow = Workflow(jobs, dependencies, root_group)
  
    return workflow

  

  def serial_jobs(self, nb=5):
    
    jobs = []
    dependencies = []
    previous_job = self.job_sleep(60)
    jobs.append(previous_job)
    for i in range(0,nb):
      job = self.job_sleep(60)
      jobs.append(job)
      dependencies.append((previous_job, job))
      previous_job = job
 
    workflow = Workflow(jobs, dependencies)
  
    return workflow


  def fake_pipelineT1(self):
    jobs = []
    dependencies = []
    root_group = []
    for i in range(0, 100):
      job1 = self.job_sleep(60) 
      job1.name = "Brain extraction"
      jobs.append(job1)

      job11 = self.job_sleep(1) 
      job11.name = "test 1"
      jobs.append(job11)
      job12 = self.job_sleep(1) 
      job12.name = "test 2"
      jobs.append(job12)
      job13 = self.job_sleep(1) 
      job13.name = "test 3"
      jobs.append(job13)

      job2 = self.job_sleep(120) 
      job2.name ="Gray/white segmentation"
      jobs.append(job2)
      job3 = self.job_sleep(400) 
      job3.name = "Left hemisphere sulci recognition"
      jobs.append(job3)
      job4 = self.job_sleep(400) 
      job4.name = "Right hemisphere sulci recognition"
      jobs.append(job4)

      #dependencies.append((job1, job2))
      dependencies.append((job1, job11))
      dependencies.append((job11, job12))
      dependencies.append((job12, job13))
      dependencies.append((job13, job2))
      dependencies.append((job2, job3))
      dependencies.append((job2, job4))
      
      group_sulci = Group(name="Sulci recognition", 
                          elements=[job3, job4])
      group_subject = Group(name="sulci recognition -- subject "+ repr(i), 
                            elements=[job1, job11, job12, job13, job2, group_sulci])

      root_group.append(group_subject)
   
    workflow = Workflow(jobs, dependencies, root_group)

    return workflow

class WorkflowTest(unittest.TestCase):
  
  LOCAL_PATH = "local path"
  FILE_TRANSFER = "file transfer"
  SHARED_RESOURCE_PATH = "shared resource path"

  wf_ctrl = None

  path_management = None

  wf_examples = None

  wf_id = None

  @classmethod
  def setup_wf_controller(cls, workflow_controller):
    cls.wf_ctrl = workflow_controller

  @classmethod
  def setup_path_management(cls, path_management):
    '''
    * path_management: LOCAL_PATH, FILE_TRANSFER or SHARED_RESOURCE_PATH
    '''
    cls.path_management = path_management

  def setUp(self):  
    if WorkflowTest.path_management == WorkflowTest.LOCAL_PATH:
      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=False,
                                          with_shared_resource_path=False)
    elif WorkflowTest.path_management == WorkflowTest.FILE_TRANSFER: 
      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=True,
                                          with_shared_resource_path=False)
    elif WorkflowTest.path_management == WorkflowTest.SHARED_RESOURCE_PATH:
      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=False,
                                          with_shared_resource_path=True)
    #raise Exception("WorkflowTest is an abstract class.")

  def tearDown(self): 
    if self.wf_id:
      WorkflowTest.wf_ctrl.delete_workflow(self.wf_id)


class MultipleTest(WorkflowTest):

  def test_result(self):
    workflow = WorkflowTest.wf_examples.multiple_simple_example() 
    self.wf_id = WorkflowTest.wf_ctrl.submit_workflow(workflow=workflow, 
                                             name="unit test multiple")
    if self.path_management == WorkflowTest.FILE_TRANSFER:
      Helper.transfer_input_files(self.wf_id, WorkflowTest.wf_ctrl)

    Helper.wait_workflow(self.wf_id, WorkflowTest.wf_ctrl)
    
    if self.path_management == WorkflowTest.FILE_TRANSFER:
      Helper.transfer_output_files(self.wf_id, WorkflowTest.wf_ctrl)
    
    status = self.wf_ctrl.workflow_status(self.wf_id)

    fail_jobs=Helper.list_failed_jobs(self.wf_id, WorkflowTest.wf_ctrl)
    
#    num_list_fail_jobs=len(fail_jobs)
#    print "num_list_fail_jobs=" + repr(num_list_fail_jobs)
#    for fail_job_id in fail_jobs:
#        print "fail job id :" +repr(fail_job_id)+"\n"
    

                                             
    (jobs_info, 
    transfers_info, 
    workflow_status, 
    workflow_queue) = WorkflowTest.wf_ctrl.workflow_elements_status(self.wf_id)
#    print "len(jobs_info)=" + repr(len(jobs_info)) + "\n"
                                             
                                             
    # TODO: check the stdout and stderrr
    for (job_id, tmp_status, queue, exit_info, dates) in jobs_info:
#        print "job_id="         +repr(job_id)+"\n"
        job_list=self.wf_ctrl.jobs([job_id])
        #print 'len(job_list)='+repr(len(job_list))+"\n"
        job_name, job_command, job_submission_date=job_list[job_id]
        
#        print "name="			+repr(job_name)+"\n"
#        print "command="        +repr(job_command)+"\n"
#        print "submission="     +repr(job_submission_date)+"\n"
#        print "tmp_status="     +repr(tmp_status)+"\n"
#        print "exit_info="		+repr(exit_info)+"\n"
#        print "dates="			+repr(dates)+"\n"
        
        
        ##To check job standard out 
        if repr(job_name)=="'job1'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)
            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout1)
            self.failUnless(isSame == True)

            if self.path_management==WorkflowTest.LOCAL_PATH:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_out_model_file11,	WorkflowTest.wf_examples.lo_file11)
                self.failUnless(isSame == True)
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_out_model_file12,	WorkflowTest.wf_examples.lo_file12)
                self.failUnless(isSame == True)
                isSame,	msg	= identicalFiles(job_stderr_file,WorkflowTest.wf_examples.lo_stderr1)
                self.failUnless(isSame == True)
            if self.path_management==WorkflowTest.FILE_TRANSFER:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_out_model_file11,	WorkflowTest.wf_examples.tr_file11.client_path)
                self.failUnless(isSame == True)    
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_out_model_file12,	WorkflowTest.wf_examples.tr_file12.client_path)
                self.failUnless(isSame == True)
                #For unknown reason, it raise some errors
                #http://stackoverflow.com/questions/10496758/unexpected-end-of-file-and-error-importing-function-definition-error-running 
                #isSame,	msg	= identicalFiles(job_stderr_file,WorkflowTest.wf_examples.lo_stderr1)
                #self.failUnless(isSame == True)
        
        if repr(job_name)=="'job1 with exception'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)
            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout1_exception_model)
            self.failUnless(isSame == True)

        if repr(job_name)=="'job2'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)
            if self.path_management==WorkflowTest.FILE_TRANSFER:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.tr_file2.client_path,	WorkflowTest.wf_examples.lo_out_model_file2)
            if self.path_management==WorkflowTest.LOCAL_PATH:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_file2,	WorkflowTest.wf_examples.lo_out_model_file2)
                self.failUnless(isSame == True)
                isSame,	msg	= identicalFiles(job_stderr_file,WorkflowTest.wf_examples.lo_stderr2)
                self.failUnless(isSame == True)
            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout2)
            self.failUnless(isSame == True)
            
            
        if repr(job_name)=="'job3'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)

            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout3)
            self.failUnless(isSame == True)
            if self.path_management==WorkflowTest.LOCAL_PATH:
                isSame,	msg	= identicalFiles(job_stderr_file,WorkflowTest.wf_examples.lo_stderr3)
                self.failUnless(isSame == True)
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_file3,	WorkflowTest.wf_examples.lo_out_model_file3)
                self.failUnless(isSame == True)
            if self.path_management==WorkflowTest.FILE_TRANSFER:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.tr_file3.client_path,	WorkflowTest.wf_examples.lo_out_model_file3)
                self.failUnless(isSame == True)
         
        if repr(job_name)=="'job4'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)
            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout4)
            self.failUnless(isSame == True)
            if self.path_management==WorkflowTest.LOCAL_PATH:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.lo_file4,	WorkflowTest.wf_examples.lo_out_model_file4)
                self.failUnless(isSame == True)
                isSame,	msg	= identicalFiles(job_stderr_file,WorkflowTest.wf_examples.lo_stderr4)
                self.failUnless(isSame == True)    
            if self.path_management==WorkflowTest.FILE_TRANSFER:
                isSame,	msg	= identicalFiles(WorkflowTest.wf_examples.tr_file4.client_path,	WorkflowTest.wf_examples.lo_out_model_file4)
                self.failUnless(isSame == True)

        if repr(job_name)=="'job3 with exception'" and exit_info[0]==constants.FINISHED_REGULARLY:
            #print "Verify "+repr(job_name)+" \n"
            job_stdout_file="/tmp/job_soma_out_log_"+repr(job_id)
            job_stderr_file="/tmp/job_soma_outerr_log_"+repr(job_id)
            self.wf_ctrl.retrieve_job_stdouterr(job_id,job_stdout_file,job_stderr_file)
            isSame,	msg	= identicalFiles(job_stdout_file,WorkflowTest.wf_examples.lo_stdout1_exception_model)
            self.failUnless(isSame == True)

    self.assert_(status == constants.WORKFLOW_DONE)
    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
                                             WorkflowTest.wf_ctrl)) == 2)
    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
                                             WorkflowTest.wf_ctrl,
                                             include_aborted_jobs=True)) == 6)

class SpecialCommandTest(WorkflowTest):

  def test_result(self):
    workflow = WorkflowTest.wf_examples.special_command() 
    self.wf_id = WorkflowTest.wf_ctrl.submit_workflow(workflow=workflow, 
                                             name="unit test command check")
    if self.path_management == WorkflowTest.FILE_TRANSFER:
      Helper.transfer_input_files(self.wf_id, WorkflowTest.wf_ctrl)

    Helper.wait_workflow(self.wf_id, WorkflowTest.wf_ctrl)
    status = self.wf_ctrl.workflow_status(self.wf_id)
    
    self.assert_(status == constants.WORKFLOW_DONE)
    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
                                             WorkflowTest.wf_ctrl)) == 0)
    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
                                             WorkflowTest.wf_ctrl,
                                             include_aborted_jobs=True)) == 0)
    # TODO: check the stdout 
 

if __name__ == '__main__':
  
  job_examples_dir = os.environ.get("SOMA_WORKFLOW_EXAMPLES")
  output_dir = os.environ.get("SOMA_WORKFLOW_EXAMPLES_OUT")
  if not job_examples_dir or not output_dir:
    raise RuntimeError( 'The environment variables SOMA_WORKFLOW_EXAMPLES and SOMA_WORKFLOW_EXAMPLES_OUT must be set.')
     
  sys.stdout.write("----- soma-workflow tests: WORKFLOW -------------\n")

  config_file_path = Configuration.search_config_path()
  sys.stdout.write("Configuration file: " + repr(config_file_path))
  resource_ids = Configuration.get_configured_resources(config_file_path)
  
  # Resource
  sys.stdout.write("Configured resources:\n")
  for i in range(0, len(resource_ids)):
    sys.stdout.write("  " + repr(i) + " -> " + repr(resource_ids[i]) + "\n")
  sys.stdout.write("Select a resource number: ")
  resource_index = int(sys.stdin.readline())
  resource_id = resource_ids[resource_index]
  sys.stdout.write("Selected resource => " + repr(resource_id) + "\n")
  sys.stdout.write("---------------------------------\n")
  login = None
  password = None
  
  config = Configuration.load_from_file(resource_id, config_file_path)

  if config.get_mode() == 'remote':
    sys.stdout.write("This is a remote connection\n")
    sys.stdout.write("login:")
    login = sys.stdin.readline()
    login = login.rstrip()
    password = sys.stdin.readline()
    password = password.rstrip()
    #password = getpass.getpass()
  sys.stdout.write("Login => " + repr(login) + "\n")
  sys.stdout.write("---------------------------------\n")
  
  # Workflow types
  wf_types = ["multiple", "special command"]
  sys.stdout.write("Workflow example to test: \n")
  sys.stdout.write("all -> all \n")
  for i in range(0, len(wf_types)):
    sys.stdout.write("  " + repr(i) + " -> " + repr(wf_types[i]) + "\n")
  sys.stdout.write("Select one or several workflow(s) : \n")
  selected_wf_type_indexes = []
  line = sys.stdin.readline()
  line = line.rstrip()
  if line == "all":
    selected_wf_type = wf_types
    sys.stdout.write("Selected workflow(s): all \n")
  else:
    for strindex in line.split(" "):
      selected_wf_type_indexes.append(int(strindex))
    selected_wf_type = []
    sys.stdout.write("Selected workflow(s): \n" )
    for wf_type_index in selected_wf_type_indexes:
      selected_wf_type.append(wf_types[int(wf_type_index)])
      sys.stdout.write("  => " + repr(wf_types[int(wf_type_index)])  + "\n")

  # File path type 
  sys.stdout.write("---------------------------------\n")
  path_types = [WorkflowTest.LOCAL_PATH, 
                WorkflowTest.FILE_TRANSFER,
                WorkflowTest.SHARED_RESOURCE_PATH]
  sys.stdout.write("File path type: \n")
  sys.stdout.write("all -> all \n")
  for i in range(0, len(path_types)):
    sys.stdout.write("  " + repr(i) + " -> " + repr(path_types[i]) + "\n")
  sys.stdout.write("Select one or several path type : \n")
  selected_path_type_indexes = []
  line = sys.stdin.readline()
  line = line.rstrip()
  if line == "all":
    selected_path_type = path_types
    sys.stdout.write("Selected path type: all \n")
  else:
    for strindex in line.split(" "):
      selected_path_type_indexes.append(int(strindex))
    selected_path_type = []
    sys.stdout.write("Selected path types: \n")
    for path_type_index in selected_path_type_indexes:
      selected_path_type.append(path_types[int(path_type_index)])
      sys.stdout.write("  => " + repr(path_types[int(path_type_index)])  + "\n")

  # Test type
  sys.stdout.write("---------------------------------\n")
  test_types = ["test_result"]#, "test_workflows", "test_stop", "test_restart"]
  sys.stdout.write("Tests to perform: \n")
  sys.stdout.write("all -> all \n")
  for i in range(0, len(test_types)):
    sys.stdout.write("  " + repr(i) + " -> " + repr(test_types[i]) + "\n")
  sys.stdout.write("Select one or several test : \n")
  selected_test_type_indexes = []
  line = sys.stdin.readline()
  line = line.rstrip()
  if line == "all":
    selected_test_type = test_types
    sys.stdout.write("Selected test types: all \n")
  else:
    for strindex in line.split(" "):
      selected_test_type_indexes.append(int(strindex))
    selected_test_type = []
    sys.stdout.write("Selected test types: \n")
    for test_type_index in selected_test_type_indexes:
      selected_test_type.append(test_types[int(test_type_index)])
      sys.stdout.write("  => " + repr(test_types[int(test_type_index)])  + "\n")
  sys.stdout.write("---------------------------------\n")

  wf_controller = WorkflowController(resource_id, login, password)
  WorkflowTest.setup_wf_controller(wf_controller)


  test_results = {}

  for path_management in selected_path_type:
    print "\n**********************************************************************"
    print "Test Suite: " + repr(path_management) + ":\n"
    WorkflowTest.setup_path_management(path_management)

    suite_list =  []
    tests = selected_test_type

    if "multiple" in selected_wf_type:
      suite_list.append(unittest.TestSuite(map(MultipleTest, 
                                               selected_test_type)))
    if "special command" in selected_wf_type:
      suite_list.append(unittest.TestSuite(map(SpecialCommandTest, 
                                               selected_test_type)))

    alltests = unittest.TestSuite(suite_list)
    test_result = unittest.TextTestRunner(verbosity=2).run(alltests)
    test_results[path_management] = test_result

  if len(test_results) > 1:
    for path_management, test_result in test_results.iteritems():
      if not test_result.wasSuccessful():
        print repr(path_management) + " test suite failed !"

  sys.exit(0)





'''
@author: Soizic Laguitton

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''

import os
import hashlib
import stat
import operator
import shutil
import time

class RemoteFileController(object):

  def create_file(self, path):
    f = open(path, 'wb')
    f.close()

  def write(self, path, data):
    f = open(path, 'ab')
    f.write(data)
    fs = f.tell()
    f.close()
    return fs

  def read(self, path, location, buffer_size):
    f = open(path, 'rb')
    f.seek(location)
    data = f.read(buffer_size)
    f.close()
    return data
    
  def get_file_size(self, path):
    if os.path.isfile(path):
      size = os.path.getsize(path)
    else:
      size = 0
    return size

  def is_file(self, path):
    return os.path.isfile(path)

  def is_dir(self, path):
    return os.path.isdir(path)

  def get_dir_size(self, path):
    return Transfer.get_dir_size(path)

  def get_md5_hash(self, path):
    md5_hash = hashlib.md5(open(path, 'rb').read()).hexdigest()
    return md5_hash

  def top_down_dir_list(self, path):
    return Transfer.top_down_dir_list(path)

  def create_dir_structure(self, path, top_down_relalive_path):
    return Transfer.create_dir_structure(path, top_down_relalive_path)

  def create_dirs(self, path):
    if not os.path.isdir(os.path.dirname(path)):
      os.makedirs(os.path.dirname(path))



class TransferMonitoring(object):
  
  remote_file_controller = None

  def __init__(self, remote_file_controller):
    self.remote_file_controller = remote_file_controller

    
  def transfer_to_remote_progression(self, path, remote_path):
    '''
    returns a tuple (data size, size of data transfered)
    '''
    #print ">>progress " + repr(path) + " to " + repr(remote_path)
    if os.path.isfile(path):
      r_size = self.remote_file_controller.get_file_size(remote_path)
      size = os.path.getsize(path)
    elif os.path.isdir(path):
      r_size = self.remote_file_controller.get_dir_size(remote_path)
      size = self.get_dir_size(path)
    else:
      r_size = 0
      size = 0
    #print "<<progress " + repr(r_size) + " " + repr(size)
    return (size, r_size)


  def transfer_from_remote_progression(self, remote_path, path):
    '''
    returns a tuple (data size, size of data transfered)
    '''
    #print ">>progress " + repr(remote_path) + " to " + repr(path)
    if self.remote_file_controller.is_file(remote_path):
      r_size = self.remote_file_controller.get_file_size(remote_path)
      if os.path.isfile(path):
        size = os.path.getsize(path)
      else:
        size = 0
    elif self.remote_file_controller.is_dir(remote_path):
      r_size = self.remote_file_controller.get_dir_size(remote_path)
      if os.path.isdir(path):
        size = self.get_dir_size(path)
      else:
        size = 0
    else:
      r_size = 0
      size = 0
    #print "<<progress " + repr(r_size) + " " + repr(size)
    return (r_size, size)


  def get_dir_size(self, path):
    return Transfer.get_dir_size(path)
    



class Transfer(object):

  remote_file_controller = None

  def __init__(self, remote_file_controller):
    self.remote_file_controller = remote_file_controller

  def transfer_to_remote(self, path, remote_path):
    '''
    Transfer a file or a directory to a remote location.

    * path *string*
      Path of the local file or directory

    * remote_path *string*
      Path on the remote file system.
    '''
    pass

  def transfer_from_remote(self, remote_path, path):
    '''
    Transfer a file or a directory from a remote location.

    * path *string*
      Path of the local file or directory

    * remote_path *string*
      Path on the remote file system.

    '''

    pass


  @staticmethod
  def get_dir_size(path):
    if not os.path.isdir(path):
      return 0
    size = 0
    for (directory, dirs, files) in os.walk(path):
      for file in files:
        path = os.path.join(directory, file)
        file_size = os.path.getsize(path)
        size = size + file_size
        #print "size: %0.1f MB cumul: %0.1f MB" %(file_size/(1024*1024.0), size/(1024*1024.0))
    return size

  @staticmethod
  def top_down_dir_list(path):
    abs_path = os.path.abspath(path)
    dir_list = []
    file_path_dict = {}
    for root, dirs, files in os.walk(abs_path):
      r_root = root[len(abs_path)+1:]
      if r_root:
        dir_list.append(r_root)
        file_list = []
        for name in files:
          file_list.append(name)
        file_path_dict[r_root] = file_list
    return (dir_list, file_path_dict)

  @staticmethod
  def create_dir_structure(path, top_down_relalive_path):
    if not os.path.isdir(path):
      os.mkdir(path)
    for dir_path in top_down_relalive_path:
      abs_path = os.path.join(path, dir_path)
      if not os.path.isdir(abs_path):
        os.mkdir(abs_path)


class TransferSCP(Transfer):

  username = None

  hostname = None

  def __init__(self, remote_file_controller, username, hostname):
    super(TransferSCP, self).__init__(remote_file_controller)
    self.username = username
    self.hostname = hostname    
    #print "SCP transfer"

  def transfer_to_remote(self, path, remote_path):
    if os.path.isfile(path):
      self.remote_file_controller.create_dirs(remote_path)
      if self.username != None and self.hostname != None:
        scp_cmd = 'scp -qp %s "%s@%s:%s"' %(path, 
                                            self.username, 
                                            self.hostname, 
                                            remote_path)
      else:
        scp_cmd = 'scp -qp %s %s' %(path, remote_path)
      print scp_cmd
      os.system(scp_cmd)

    if os.path.isdir(path):
      self.remote_file_controller.create_dirs(remote_path)
      if self.remote_file_controller.is_dir(remote_path):
        if self.username != None and self.hostname != None:
          scp_cmd = 'scp -Cqpr %s "%s@%s:%s"' %(os.path.join(path,"*"), 
                                                self.username, 
                                                self.hostname, 
                                                remote_path)
        else:
          scp_cmd = 'scp -Cqpr %s %s' %(os.path.join(path, "*"), remote_path)
      else:
        if self.username != None and self.hostname != None:
          scp_cmd = 'scp -Cqpr %s "%s@%s:%s"' %(path, 
                                                self.username, 
                                                self.hostname, 
                                                remote_path)
        else:
          scp_cmd = 'scp -Cqpr %s %s' %(path, remote_path)
      print scp_cmd
      os.system(scp_cmd)
      

  def transfer_from_remote(self, remote_path, path):
    if self.remote_file_controller.is_file(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      if self.username != None and self.hostname != None:
        scp_cmd = 'scp -qp "%s@%s:%s" %s ' %(self.username, 
                                            self.hostname, 
                                            remote_path, 
                                            path)
      else:
        scp_cmd = 'scp -qp %s %s ' %(remote_path, path)
      print scp_cmd
      os.system(scp_cmd)
      
    if self.remote_file_controller.is_dir(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      if os.path.isdir(path):
        if self.username != None and self.hostname != None:
          scp_cmd = 'scp -Cqpr "%s@%s:%s" %s ' %(self.username, 
                                                self.hostname, 
                                                os.path.join(remote_path, "*"), 
                                                path)
        else:
          scp_cmd = 'scp -Cqpr %s %s ' %(os.path.join(remote_path, "*"), path)
      else:
        if self.username != None and self.hostname != None:
          scp_cmd = 'scp -Cqpr "%s@%s:%s" %s ' %(self.username, 
                                                self.hostname, 
                                                remote_path, 
                                                path)
        else:
          scp_cmd = 'scp -Cqpr %s %s ' %(remote_path, path)

      print scp_cmd
      os.system(scp_cmd)
      

class TransferRsync(Transfer):

  username = None

  hostname = None

  def __init__(self, remote_file_controller, username, hostname):
    super(TransferRsync, self).__init__(remote_file_controller)
    self.username = username
    self.hostname = hostname
    #print "Rsync transfer"


  def transfer_to_remote(self, path, remote_path):
    if os.path.isfile(path):
      self.remote_file_controller.create_dirs(remote_path)
      if self.username != None and self.hostname != None:
        rsync_cmd = 'rsync -qp %s "%s@%s:%s"' %(path, 
                                            self.username, 
                                            self.hostname, 
                                            remote_path)
      else:
        rsync_cmd = 'rsync -qp %s %s' %(path, remote_path)
      print rsync_cmd
      os.system(rsync_cmd)

    if os.path.isdir(path):
      self.remote_file_controller.create_dirs(remote_path)
      if self.username != None and self.hostname != None:
        rsync_cmd = 'rsync -qpr %s "%s@%s:%s"' %(os.path.join(path, "*"), 
                                              self.username, 
                                              self.hostname, 
                                              remote_path)
      else:
        rsync_cmd = 'rsync -qpr %s %s' %(os.path.join(path, "*"), 
                                         remote_path)
      print rsync_cmd
      os.system(rsync_cmd)
      

  def transfer_from_remote(self, remote_path, path):
    if self.remote_file_controller.is_file(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      if self.username != None and self.hostname != None:
        rsync_cmd = 'rsync -qp "%s@%s:%s" %s ' %(self.username, 
                                            self.hostname, 
                                            remote_path, 
                                            path)
      else:
        rsync_cmd = 'rsync -qp %s %s ' %(remote_path, path)
      print rsync_cmd
      os.system(rsync_cmd)
      
    if self.remote_file_controller.is_dir(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      if self.username != None and self.hostname != None:
        rsync_cmd = 'rsync -qpr "%s@%s:%s" %s ' %(self.username, 
                                              self.hostname, 
                                              os.path.join(remote_path, "*"), 
                                              path)
      else:
        rsync_cmd = 'rsync -qpr %s %s ' %(os.path.join(remote_path, "*"), 
                                          path)
      print rsync_cmd
      os.system(rsync_cmd)

  
class TransferLocal(Transfer):
   
  def __init__(self, remote_file_controller):
    super(TransferLocal, self).__init__(remote_file_controller)
    #print "Local transfer"

  def transfer_to_remote(self, path, remote_path):
    #print "copy " + repr(path) + " to " + repr(remote_path)
    #time.sleep(4)
    if os.path.isfile(path):
      if not os.path.isdir(os.path.dirname(remote_path)):
        os.makedirs(os.path.dirname(remote_path))
      shutil.copy(path, remote_path)

    if os.path.isdir(path):
      if not os.path.isdir(os.path.dirname(remote_path)):
        os.makedirs(os.path.dirname(remote_path))
      if os.path.isdir(remote_path):
        for p in os.listdir(path):
          self.transfer_to_remote(os.path.join(path, p), 
                                  os.path.join(remote_path, p))
      else:
        shutil.copytree(path, remote_path)
    #time.sleep(4)
      

  def transfer_from_remote(self, remote_path, path):
    #print "copy " + repr(remote_path) + " to " + repr(path)
    #time.sleep(4)
    if os.path.isfile(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      shutil.copy(remote_path, path)
      
    if os.path.isdir(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        #print "makedirs " + repr(os.path.dirname(path))
        os.makedirs(os.path.dirname(path))
      if os.path.isdir(path):
        for p in os.listdir(remote_path):
          self.transfer_from_remote(os.path.join(remote_path, p),
                                    os.path.join(path, p))
      else:
        shutil.copytree(remote_path, path)
    #time.sleep(4)


class PortableRemoteTransfer(Transfer):

  def __init__(self, remote_file_controller):
    super(PortableRemoteTransfer, self).__init__(remote_file_controller)
    #print "Portable transfer"

  def transfer_to_remote(self, 
                         path, 
                         remote_path,
                         buffer_size=512**2):
    '''
    return Transfered_with_success
    '''
    print "copy " + repr(path) + " to " + repr(remote_path)
    if os.path.isfile(path):
      self.remote_file_controller.create_dirs(remote_path)
      # TBI in case the file were already transfered
      transmitted = 0

      f = open(path, 'rb')
      self.remote_file_controller.create_file(remote_path)
      file_size = os.path.getsize(path)
      if transmitted:
        f.seek(transmitted)
      r_file_size = transmitted
      while r_file_size < file_size:
        r_file_size = self.remote_file_controller.write(remote_path,   
                                                        f.read(buffer_size))
      f.close()
      
      if r_file_size != file_size:
        pass
        #TBI error

      md5_hash = hashlib.md5(open(path, 'rb').read()).hexdigest()
      if md5_hash != self.remote_file_controller.get_md5_hash(remote_path):
        #TBI error
        pass


    elif os.path.isdir(path):
      self.remote_file_controller.create_dirs(remote_path)

      (dir_list, file_path_dict) = self.top_down_dir_list(path)
      self.remote_file_controller.create_dir_structure(remote_path,
                                                       dir_list)
      for relative_dir_path, file_list in file_path_dict.iteritems():
        dir_path = os.path.join(path, relative_dir_path)
        r_dir_path = os.path.join(remote_path, relative_dir_path)
        for file_name in file_list: 
          remote_file_path = os.path.join(r_dir_path, file_name)
          file_path = os.path.join(dir_path, file_name)
          self.transfer_to_remote(file_path, 
                                  remote_file_path,
                                  buffer_size=buffer_size)
 
    

  def transfer_from_remote(self, 
                           remote_path, 
                           path,
                           buffer_size = 512**2):

   print "copy " + repr(remote_path) + " to " + repr(path)
   if self.remote_file_controller.is_file(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
      # TBI in case the file were already transfered
      transmitted = 0

      if transmitted:
        f = open(path, 'ab')
      else:
        f = open(path, 'wb')

      remote_file_size = self.remote_file_controller.get_file_size(remote_path)

      transmitted = os.path.getsize(path)
      data = self.remote_file_controller.read(remote_path,
                                              transmitted,
                                              buffer_size)
      f.write(data)
      file_size = f.tell()
      while data and file_size < remote_file_size:
        data = self.remote_file_controller.read(remote_path,
                                                file_size,
                                                buffer_size)
        f.write(data)
        file_size = f.tell()
      
      f.close()
      

      if file_size != remote_file_size:
        pass
        #TBI error

      md5_hash = hashlib.md5(open(path, 'rb').read()).hexdigest()
      if md5_hash != self.remote_file_controller.get_md5_hash(remote_path):
        #TBI error
        pass


   elif self.remote_file_controller.is_dir(remote_path):
      if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))

      (dir_list, file_path_dict) = self.remote_file_controller.top_down_dir_list(remote_path)
      self.create_dir_structure(path,
                                dir_list)
      for relative_dir_path, file_list in file_path_dict.iteritems():
        dir_path = os.path.join(path, relative_dir_path)
        r_dir_path = os.path.join(remote_path, relative_dir_path)
        for file_name in file_list: 
          file_path = os.path.join(dir_path, file_name)
          r_file_path = os.path.join(r_dir_path, file_name)
          self.transfer_from_remote(r_file_path,
                                    file_path,
                                    buffer_size=buffer_size)
    

  def top_down_dir_list(self, path):
    return Transfer.top_down_dir_list(path)

  def create_dir_structure(self, path, top_down_relalive_path):
    return Transfer.create_dir_structure(path, top_down_relalive_path)





from __future__ import with_statement

'''
@author: Soizic Laguitton

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''


#-------------------------------------------------------------------------------
# Imports
#-------------------------------------------------------------------------

from datetime import datetime
from datetime import timedelta
import threading
import time
import socket
import getpass
import os
import select
import SocketServer


from soma_workflow.errors import ConnectionError



def ReadOutput(stdout, tag=None, num_line_stdout=-1):
    is_limit_stdout = False

    if num_line_stdout != -1:
        is_limit_stdout = True

    std_out_lines = list()

    if (is_limit_stdout == False
       or (is_limit_stdout == True and num_line_stdout != 0)
            ):
        if tag:
            sline = stdout.readline()
            while sline and sline != tag+'\n':
                sline = stdout.readline()

        sline = stdout.readline()
        while (sline and num_line_stdout != 0):

            std_out_lines.append(sline.strip())
            num_line_stdout = num_line_stdout-1
            if(num_line_stdout == 0):
                break

            sline = stdout.readline()

    return std_out_lines


def SSHExecCmd(sshcommand,
               userid,
               ip_address_or_domain,
               userpw='',
               wait_output=True,
               sshport=22,
               isNeedErr=False,
               num_line_stdout=-1,  # How many stdout or stderr lines to read
               num_line_stderr=-1):  # -1 means unlimited

    if wait_output:
        tag = '----xxxx=====start to exec=====xxxxx----'
        sshcommand = "echo %s && %s" % (tag, sshcommand)

    
    is_limit_stdout = False
    is_limit_stderr = False

    if num_line_stdout != -1:
        is_limit_stdout = True
    if num_line_stderr != -1:
        is_limit_stderr = True

    stdin = []
    stdout = []
    stderr = []

    std_out_lines = []
    std_err_lines = []

    import paramiko

    try:
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        client.load_system_host_keys()

        client.connect(hostname=ip_address_or_domain,
                       port=sshport,
                       username=userid,
                       password=userpw)

        stdin, stdout, stderr = client.exec_command(sshcommand)

    except paramiko.AuthenticationException, e:
        print   "The authentification failed. %s. " \
                "Please check your user and password. " \
                "You can test the connection in terminal with " \
                "command: ssh -p %s %s@%s" % (
                    e, sshport, userid, ip_address_or_domain)
        raise e
    except Exception, e:
        print   "Can not use ssh to log on the remote machine. "\
                "Please Make sure your network can be connected %s. "\
                "You can test the connection in terminal with "\
                "command: ssh -p %s %s@%s" % (
                    e, sshport, userid, ip_address_or_domain)
        raise e

    if wait_output:
        std_out_lines = ReadOutput(stdout, tag, num_line_stdout)
        if isNeedErr:
            std_err_lines = ReadOutput(stderr, None, num_line_stderr)

    client.close()

    if isNeedErr:
        return (std_out_lines, std_err_lines)
    else:
        return std_out_lines


def check_if_soma_wf_cr_on_server(
                            userid,
                            ip_address_or_domain,
                            userpw='',
                            sshport=22):
    command = "python -c 'import soma_workflow.check_requirement'"
    (std_out_lines, std_err_lines) = SSHExecCmd(
        command, 
        userid, 
        ip_address_or_domain, 
        userpw, 
        wait_output=True, 
        isNeedErr=True,
        sshport=sshport)

    if len(std_err_lines) > 0:
        return False
    
    return True
    
def check_if_ctype_drmaa_on_server(
                            userid,
                            ip_address_or_domain,
                            userpw='',
                            sshport=22):
    command = "python -m 'soma_workflow.check_requirement.drmaa'"
    (std_out_lines, std_err_lines) = SSHExecCmd(
        command, 
        userid, 
        ip_address_or_domain, 
        userpw, 
        wait_output=True, 
        isNeedErr=True,
        sshport=sshport)

    # print repr(std_out_lines)
    
    if len(std_out_lines) == 1:
        if std_out_lines[0] == "True":
            return True
    
    return False
    
def check_if_somawf_on_server(
                            userid,
                            ip_address_or_domain,
                            userpw='',
                            sshport=22):
    command = "python -c 'import soma_workflow'"
    (std_out_lines, std_err_lines) = SSHExecCmd(
        command, 
        userid, 
        ip_address_or_domain, 
        userpw, 
        wait_output=True, 
        isNeedErr=True,
        sshport=sshport)

    if len(std_err_lines) > 0:
        return False
    
    return True

def check_if_somawfdb_on_server(
                            ResName,
                            userid,
                            ip_address_or_domain,
                            userpw='',
                            sshport=22):

    command = "ps -ef | grep 'python -m soma_workflow.start_database_server'"\
    " | grep '%s' | grep -v grep | awk '{print $2}'" % (userid)

    std_out_lines = SSHExecCmd(
        command, 
        userid, 
        ip_address_or_domain, 
        userpw, 
        wait_output=True, 
        sshport=sshport)

    if len(std_out_lines) == 0:
        return False
    else:
        return True


def searchAvailablePort():
    s = socket.socket(
        socket.AF_INET, socket.SOCK_STREAM)  # Create a TCP socket
    s.bind(('localhost', 0))  # try to bind to the port 0 so that the system
                            # will find an available port
    available_port = s.getsockname()[1]
    s.close()
    return available_port


#-------------------------------------------------------------------------------
# Classes and functions
#-------------------------------------------------------------------------


class RemoteConnection(object):
    '''
    Remote version of the connection.
    The WorkflowControler object is created using ssh with paramiko.
    The communication between the client and the computing resource is done with
    Pyro inside a ssh port forwarding tunnel.
    '''

    def __init__(self,
                 login,
                 password,
                 cluster_address,
                 submitting_machine,
                 resource_id,
                 log="",
                 rsa_key_pass=None):
        '''
        @type  login: string
        @param login: user's login on the computing resource
        @type  password: string
        @param password: associted password
        @type  submitting_machine: string
        @param submitting_machine: address of a submitting machine of the computing
                                   resource.
        '''

        # required in the remote connection mode
        import paramiko
        from paramiko.file import BufferedFile
        import Pyro.core
        from Pyro.errors import ConnectionClosedError

        if not login:
            raise ConnectionError("Remote connection requires a login")
        

        pyro_objet_name = "workflow_engine_" + login

        if not check_if_somawf_on_server(
                                        login, 
                                        cluster_address, 
                                        password):
            raise ConnectionError("Cannot find soma-workflow on %s ."\
                                  "Please verify if your PYTHONPATH "\
                                  "includes the soma-workflow."\
                                  % (cluster_address))

       
        if not check_if_soma_wf_cr_on_server(
                                        login, 
                                        cluster_address, 
                                        password):
            raise ConnectionError("Cannot find "\
                "soma_workflow.check_requirement on %s ."\
                "Please update your soma-workflow on %s."\
                                  % (cluster_address,cluster_address))
                                  
        if not check_if_ctype_drmaa_on_server(
                                        login, 
                                        cluster_address, 
                                        password):
            raise ConnectionError("Cannot find "\
                "drmaa libary on %s ."\
                "Please verify your drmaa libary on %s. "\
                "Or setup up enviroment variable DRMAA_LIBRARY_PATH."\
                                  % (cluster_address,cluster_address))
                                  
                                                
        if not check_if_somawfdb_on_server(
                                            resource_id, 
                                            login, 
                                            cluster_address, 
                                            password):
           command = "python -m soma_workflow.start_database_server %s & bg"%(resource_id)
           SSHExecCmd(
              command,
              login,
              cluster_address,
              userpw=password,
              wait_output=False)
 
         


        # run the workflow engine process and get back the    #
        # WorkflowEngine and ConnectionChecker URIs       #
        command = "python -m soma_workflow.start_workflow_engine"\
                  " %s %s %s" % (resource_id, pyro_objet_name, log)
        
        print "start engine command: "\
              "ssh %s@%s %s" % (login, cluster_address, command)

        (std_out_lines) = SSHExecCmd(
            command,
            login,
            cluster_address,
            userpw=password,
            wait_output=True,
            sshport=22,
            num_line_stdout=3)
        
        
        # print "std_out_lines="+repr(std_out_lines)

        workflow_engine_uri = None
        connection_checker_uri = None
        configuration_uri = None

        for std_out_line in std_out_lines:
            if std_out_line.split()[0] == pyro_objet_name:
                workflow_engine_uri = std_out_line.split()[1]
            elif std_out_line.split()[0] == "connection_checker":
                connection_checker_uri = std_out_line.split()[1]
            elif std_out_line.split()[0] == "configuration":
                configuration_uri = std_out_line.split()[1]

        if (not configuration_uri or
            not connection_checker_uri or
                not configuration_uri):
            raise ConnectionError("A problem occured while starting the engine "
                                  "process on the remote machine " +
                                  repr(cluster_address) + "\n"
                                  "**More details:**\n"
                                  "**Start engine process command line:** \n"
                                  "\n" + command + "\n\n"
                                  "**Engine process standard output:** \n"
                                  "\n" + repr(std_out_lines))

        # print "workflow_engine_uri: " +  workflow_engine_uri
        # print "connection_checker_uri: " +  connection_checker_uri
        # print "configuration_uri: " + configuration_uri
        engine_pyro_daemon_port = Pyro.core.processStringURI(
            workflow_engine_uri).port
        # print "Pyro object port: " + repr(engine_pyro_daemon_port)

        # find an available port              #
        client_pyro_daemon_port = searchAvailablePort()
        # print "client pyro object port: " + repr(client_pyro_daemon_port)

        # tunnel creation                      #
        try:
            self.__transport = paramiko.Transport((cluster_address, 22))
            self.__transport.setDaemon(True)
            self.__transport.connect(username=login, password=password)
            if not password:
                rsa_file_path = os.path.join(
                    os.environ['HOME'], '.ssh', 'id_rsa')
                print "reading RSA key in " + repr(rsa_file_path)
                if rsa_key_pass:
                    key = paramiko.RSAKey.from_private_key_file(
                                        rsa_file_path,
                                        password=rsa_key_pass)
                else:
                    key = paramiko.RSAKey.from_private_key_file(rsa_file_path)
                self.__transport.auth_publickey(login, key)
                # TBI DSA Key => see paramamiko/demos/demo.py for an example
            print "tunnel creation " + str(login) + "@" + cluster_address + \
                  "   port: " + repr(client_pyro_daemon_port) + " host: " + \
                  str(submitting_machine) + " host port: " + str(
                      engine_pyro_daemon_port)

            tunnel = Tunnel(client_pyro_daemon_port,
                            submitting_machine,
                            engine_pyro_daemon_port,
                            self.__transport)

            tunnel.start()

        except paramiko.AuthenticationException, e:
            raise ConnectionError("The authentification failed while "
                                  "creating the ssh tunnel. %s" % (e))
        except Exception, e:
            raise ConnectionError("The ssh communication tunnel could not be created."
                                  "%s: %s" % (type(e), e))

        # create the proxies                     #
        self.workflow_engine = Pyro.core.getProxyForURI(workflow_engine_uri)
        connection_checker = Pyro.core.getAttrProxyForURI(
            connection_checker_uri)
        self.configuration = Pyro.core.getAttrProxyForURI(configuration_uri)

        # setting the proxies to use the tunnel  #
        self.workflow_engine.URI.port = client_pyro_daemon_port
        self.workflow_engine.URI.address = 'localhost'
        connection_checker.URI.port = client_pyro_daemon_port
        connection_checker.URI.address = 'localhost'

        # waiting for the tunnel to be set
        tunnelSet = False
        maxattemps = 3
        attempts = 0
        while not tunnelSet and attempts < maxattemps:
            try:
                attempts = attempts + 1
                print "Communication through the ssh tunnel. Attempt no " + \
                repr(attempts) + "/" + repr(maxattemps)
                self.workflow_engine.jobs()
                connection_checker.isConnected()
            except Pyro.errors.ProtocolError, e:
                print "-> Communication through ssh tunnel Failed. %s: %s" \
                % (type(e), e)
                time.sleep(1)
            except Exception, e:
                print "-> Communication through ssh tunnel Failed. %s: %s" \
                % (type(e), e)
                time.sleep(1)

            else:
                print "-> Communication through ssh tunnel OK"
                tunnelSet = True

        if attempts > maxattemps:
            raise ConnectionError("The ssh tunnel could not be started within"
                                  + repr(maxattemps) + " seconds.")

        # create the connection holder objet for #
        # a clean disconnection in any case      #
        self.__connection_holder = ConnectionHolder(connection_checker)
        self.__connection_holder.start()

    def isValid(self):
        return self.__connection_holder.isAlive()

    def stop(self):
        '''
        For test purpose only !
        '''
        self.__connection_holder.stop()
        self.__transport.close()

    def get_workflow_engine(self):
        return self.workflow_engine

    def get_configuration(self):
        return self.configuration


class LocalConnection(object):

    '''
    Local version of the connection.
    The worjkflow engine process is created using subprocess.
    '''

    def __init__(self,
                 resource_id,
                 log=""):

        # required in the local connection mode
        import Pyro.core
        from Pyro.errors import ConnectionClosedError
        import subprocess

        login = getpass.getuser()
        pyro_objet_name = "workflow_engine_" + login

        # run the workflow engine process and get back the
        # workflow_engine and ConnectionChecker URIs
        # command = "python -m cProfile -o /home/soizic/profile/profile /home/soizic/svn/brainvisa/source/soma/soma-workflow/trunk/python/soma_workflow/start_workflow_engine.py %s %s %s" %(
                                         # resource_id,
                                         # pyro_objet_name,
                                         # log)
        command = "python -m soma_workflow.start_workflow_engine %s %s %s" % (
            resource_id,
            pyro_objet_name,
            log)
        # command = "rpdb2 -p Soizic -d /home/soizic/svn/brainvisa/source/soma/soma-workflow/trunk/python/soma_workflow/start_workflow_engine.py %s %s %s" %(
                                         # resource_id,
                                         # pyro_objet_name,
                                         # log)

        # print command

        engine_process = subprocess.Popen(command,
                                          shell=True,
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)

        line = engine_process.stdout.readline()
        stdout_content = line
        while line and line.split()[0] != pyro_objet_name:
            line = engine_process.stdout.readline()
            stdout_content = stdout_content + "\n" + line

        if not line:  # A problem occured while starting the engine.
            line = engine_process.stderr.readline()
            stderr_content = line
            while line:
                line = engine_process.stderr.readline()
                stderr_content = stderr_content + "\n" + line
            raise ConnectionError("A problem occured while starting the engine "
                                  "process on the local machine. \n"
                                  "**More details:**\n"
                                  "**Start engine process command line:** \n"
                                  "\n" + command + "\n\n"
                                  "**Engine process standard output:** \n"
                                  "\n" + stdout_content +
                                  "**Engine process standard error:** \n"
                                  "\n" + stderr_content)
        workflow_engine_uri = line.split()[1]
        line = engine_process.stdout.readline()
        stdout_content = stdout_content + "\n" + line
        while line and line.split()[0] != "connection_checker":
            line = engine_process.stdout.readline()
            stdout_content = stdout_content + "\n" + line

        if not line:  # A problem occured while starting the engine.
            line = engine_process.stderr.readline()
            stderr_content = line
            while line:
                line = engine_process.stderr.readline()
                stderr_content = stderr_content + "\n" + line
            raise ConnectionError("A problem occured while starting the engine "
                                  "process on the local machine. \n"
                                  "**More details:**\n"
                                  "**Start engine process command line:** \n"
                                  "\n" + command + "\n\n"
                                  "**Engine process standard output:** \n"
                                  "\n" + stdout_content +
                                  "**Engine process standard error:** \n"
                                  "\n" + stderr_content)

        connection_checker_uri = line.split()[1]
        line = engine_process.stdout.readline()
        stdout_content = stdout_content + "\n" + line
        while line and line.split()[0] != "configuration":
            line = engine_process.stdout.readline()
            stdout_content = stdout_content + "\n" + line

        if not line:  # A problem occured while starting the engine.
            line = engine_process.stderr.readline()
            stderr_content = line
            while line:
                line = engine_process.stderr.readline()
                stderr_content = stderr_content + "\n" + line
            raise ConnectionError("A problem occured while starting the engine "
                                  "process on the local machine. \n"
                                  "**More details:**\n"
                                  "**Start engine process command line:** \n"
                                  "\n" + command + "\n\n"
                                  "**Engine process standard output:** \n"
                                  "\n" + stdout_content +
                                  "**Engine process standard error:** \n"
                                  "\n" + stderr_content)
        configuration_uri = line.split()[1]

        print "workflow_engine_uri: " + workflow_engine_uri
        print "connection_checker_uri: " + connection_checker_uri
        print "configuration_uri: " + configuration_uri

        # create the proxies                     #
        self.workflow_engine = Pyro.core.getProxyForURI(workflow_engine_uri)
        connection_checker = Pyro.core.getAttrProxyForURI(
            connection_checker_uri)
        self.configuration = Pyro.core.getAttrProxyForURI(configuration_uri)

        # create the connection holder objet for #
        # a clean disconnection in any case      #
        self.__connection_holder = ConnectionHolder(connection_checker)
        self.__connection_holder.start()

    def isValid(self):
        return self.__connection_holder.isAlive()

    def stop(self):
        '''
        For test purpose only !
        '''
        self.__connection_holder.stop()

    def get_workflow_engine(self):
        return self.workflow_engine

    def get_configuration(self):
        return self.configuration


class ConnectionChecker(object):

    def __init__(self, interval=2, controlInterval=3):
        self.connected = False
        self.lock = threading.RLock()
        self.interval = timedelta(seconds=interval)
        self.controlInterval = controlInterval
        self.lastSignal = datetime.now() - timedelta(days=15)

        def controlLoop(self, control_interval):
            while True:
                with self.lock:
                    ls = self.lastSignal
                delta = datetime.now()-ls
                if delta > self.interval * 3:
                    self.disconnectionCallback()
                    self.connected = False
                else:
                    self.connected = True
                time.sleep(control_interval)

        self.controlThread = threading.Thread(name="connectionControlThread",
                                              target=controlLoop,
                                              args=(self, controlInterval))
        self.controlThread.setDaemon(True)
        self.controlThread.start()

    def signalConnectionExist(self):
        with self.lock:
            # print "ConnectionChecker <= a signal was received"
            self.lastSignal = datetime.now()

    def isConnected(self):
        return self.connected

    def disconnectionCallback(self):
        pass


class ConnectionHolder(threading.Thread):
    def __init__(self, connectionChecker):
        threading.Thread.__init__(self)
        self.setDaemon(True)
        self.name = "connectionHolderThread"
        self.connectionChecker = connectionChecker
        self.interval = self.connectionChecker.interval.seconds

    def run(self):
        from Pyro.errors import ConnectionClosedError
        self.stopped = False
        while not self.stopped:
            # print "ConnectionHolder => signal"
            try:
                self.connectionChecker.signalConnectionExist()
            except ConnectionClosedError, e:
                print "Connection closed"
                break
            time.sleep(self.interval)

    def stop(self):
        self.stopped = True


class Tunnel(threading.Thread):

    class ForwardServer (SocketServer.ThreadingTCPServer):
        daemon_threads = True
        allow_reuse_address = True

    class Handler (SocketServer.BaseRequestHandler):

        def setup(self):
            # print 'Setup : %s %d' %(repr(self.chain_host), self.chain_port)
            try:
                self.__chan = self.ssh_transport.open_channel('direct-tcpip',
                                                             (self.chain_host,
                                                              self.chain_port),
                                                              self.request.getpeername())
            except Exception, e:
                raise ConnectionError('Incoming request to %s:%d failed: %s' % (
                    self.chain_host, self.chain_port, repr(e)))

            if self.__chan is None:
                raise ConnectionError('Incoming request to %s:%d was rejected by the SSH server.' %
                                     (self.chain_host, self.chain_port))

            print 'Connected!  Tunnel open %r -> %r -> %r' % (
            self.request.getpeername(), 
            self.__chan.getpeername(), 
            (self.chain_host, self.chain_port))

        def handle(self):
            # print 'Handle : %s %d' %(repr(self.chain_host), self.chain_port)
            while True:
                r, w, x = select.select([self.request, self.__chan], [], [])
                if self.request in r:
                    data = self.request.recv(1024)
                    if len(data) == 0:
                        break
                    self.__chan.send(data)
                if self.__chan in r:
                    data = self.__chan.recv(1024)
                    if len(data) == 0:
                        break
                    self.request.send(data)

        def finish(self):
            print 'Tunnel closed from %r' % (self.request.getpeername(),)
            self.__chan.close()
            self.request.close()

    def __init__(self, port, host, hostport, transport):
        threading.Thread.__init__(self)
        self.__port = port
        self.__host = host
        self.__hostport = hostport
        self.__transport = transport
        self.setDaemon(True)

    def run(self):
        host = self.__host
        hostport = self.__hostport
        transport = self.__transport
        port = self.__port

        class SubHander (Tunnel.Handler):
            chain_host = host
            chain_port = hostport
            ssh_transport = transport
        try:
            Tunnel.ForwardServer(('', port), SubHander).serve_forever()
        except KeyboardInterrupt:
            print 'tunnel %d:%s:%d stopped !' % (port, host, hostport)
        except Exception, e:
            print 'Tunnel Error. %s: %s' % (type(e), e)



#!/usr/bin/env python

'''
@author: Soizic Laguitton

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''



if __name__=="__main__":

  import sys
  import threading
  import time 
  import logging
  import os

  import Pyro.naming
  import Pyro.core
  from Pyro.errors import PyroError, NamingError, ProtocolError
  
  import soma_workflow.engine
  import soma_workflow.scheduler
  import soma_workflow.connection 
  import soma_workflow.configuration
  from soma_workflow.errors import NoDrmaaLibError, EngineError
  from soma_workflow.database_server import WorkflowDatabaseServer



  ###### WorkflowEngine pyro object
  class ConfiguredWorkflowEngine(Pyro.core.SynchronizedObjBase, 
                       soma_workflow.engine.ConfiguredWorkflowEngine):
    def __init__(self, database_server, scheduler, config):
      Pyro.core.SynchronizedObjBase.__init__(self)
      soma_workflow.engine.ConfiguredWorkflowEngine.__init__(self, 
                                                   database_server, 
                                                   scheduler,
                                                   config)
    pass
    
  class ConnectionChecker(Pyro.core.ObjBase, 
                          soma_workflow.connection.ConnectionChecker):
    def __init__(self, interval=1, control_interval=3):
      Pyro.core.ObjBase.__init__(self)
      soma_workflow.connection.ConnectionChecker.__init__(self, 
                                                          interval, 
                                                          control_interval)
    pass


  class Configuration(Pyro.core.ObjBase, 
                      soma_workflow.configuration.Configuration):

    def __init__( self,
                  resource_id,
                  mode,
                  scheduler_type,
                  database_file,
                  transfered_file_dir,
                  submitting_machines=None,
                  cluster_address=None,
                  name_server_host=None,
                  server_name=None,
                  queues=None,
                  queue_limits=None,
                  drmaa_implementation=None):
      Pyro.core.ObjBase.__init__(self)
      soma_workflow.configuration.Configuration.__init__(self,
                                                         resource_id,
                                                         mode,
                                                         scheduler_type,
                                                         database_file,
                                                         transfered_file_dir,
                                                         submitting_machines,
                                                         cluster_address,
                                                         name_server_host,
                                                         server_name,
                                                         queues,
                                                         queue_limits,
                                                         drmaa_implementation)
      pass

  def get_database_server_proxy(config, logger):
    try:
      Pyro.core.initClient()
      locator = Pyro.naming.NameServerLocator()
      name_server_host = config.get_name_server_host() 
      if name_server_host == 'None':
        ns = locator.getNS()
      else: 
        ns = locator.getNS(host=name_server_host)
    except Exception, e:
      raise EngineError("Could not find the Pyro name server.")

    server_name = config.get_server_name()

    try:
      uri = ns.resolve(server_name)
      logger.info('Server URI:'+ repr(uri))
    except NamingError,e:
      raise EngineError("%s %s \n"   
                        "Could not find soma-workflow database server. "
                        "Run the following command on %s to start the server: \n"
                        "python -m soma_workflow.start_database_server %s" %(type(e), e, name_server_host, resource_id))

    database_server = Pyro.core.getProxyForURI(uri)
	
    try:
      database_server.test()
    except ProtocolError,e:
      raise EngineError("%s %s \n"
	                "The database server might not be running. "
                        "Run the following command on %s to start the server: \n"
                        "python -m soma_workflow.start_database_server %s" %(type(e), e, name_server_host, resource_id))
    return database_server

    
  
  ###### main server program
  def main(resource_id, engine_name, log = ""):

    config = Configuration.load_from_file(resource_id)

    (engine_log_dir,
    engine_log_format,
    engine_log_level) = config.get_engine_log_info()
    if engine_log_dir:
      logfilepath = os.path.join(os.path.abspath(engine_log_dir),  
                                 "log_" + engine_name + log)
      logging.basicConfig(
           filename=logfilepath,
           format=engine_log_format,
           level=eval("logging." + engine_log_level))
      logger = logging.getLogger('engine')
      logger.info(" ")
      logger.info("****************************************************")
      logger.info("****************************************************")
 
    if config.get_scheduler_type() == soma_workflow.configuration.DRMAA_SCHEDULER:
        
      if not soma_workflow.scheduler.DRMAA_LIB_FOUND:
          raise NoDrmaaLibError
          
      sch = soma_workflow.scheduler.DrmaaCTypes(config.get_drmaa_implementation(), 
                                    config.get_parallel_job_config(),
                                    os.path.expanduser("~"),
                                    configured_native_spec=config.get_native_specification())
      database_server = get_database_server_proxy(config, logger)

    elif config.get_scheduler_type() == soma_workflow.configuration.LOCAL_SCHEDULER:
      from soma_workflow.scheduler import ConfiguredLocalScheduler
      local_scheduler_cfg_file_path = soma_workflow.configuration.LocalSchedulerCfg.search_config_path()
      if local_scheduler_cfg_file_path:
        local_scheduler_config = soma_workflow.configuration.LocalSchedulerCfg.load_from_file(local_scheduler_cfg_file_path)
      else:
      	local_scheduler_config = soma_workflow.configuration.LocalSchedulerCfg()      
      sch = ConfiguredLocalScheduler(local_scheduler_config)
      database_server = get_database_server_proxy(config, logger)

    elif config.get_scheduler_type() == soma_workflow.configuration.MPI_SCHEDULER:
      sch = None 
      database_server = WorkflowDatabaseServer(config.get_database_file(),
                                               config.get_transfered_file_dir())


    #Pyro.config.PYRO_MULTITHREADED = 0
    Pyro.core.initServer()
    daemon = Pyro.core.Daemon()

    workflow_engine = ConfiguredWorkflowEngine(database_server, 
                                               sch,
                                               config)

    # connection to the pyro daemon and output its URI 
    uri_engine = daemon.connect(workflow_engine, engine_name)
    sys.stdout.write(engine_name + " " + str(uri_engine) + "\n")
    sys.stdout.flush()
  
    logger.info('Pyro object ' + engine_name + ' is ready.')
    
    # connection check
    connection_checker = ConnectionChecker()
    uri_cc = daemon.connect(connection_checker, 'connection_checker')
    sys.stdout.write("connection_checker " + str(uri_cc) + "\n")
    sys.stdout.flush() 

    # configuration
    uri_config = daemon.connect(config, 'configuration')
    sys.stdout.write("configuration " + str(uri_config) + "\n")
    sys.stdout.flush() 
    
    # Daemon request loop thread
    logger.info("daemon port = " + repr(daemon.port))
    daemon_request_loop_thread = threading.Thread(name="pyro_request_loop", 
                                                  target=daemon.requestLoop) 
  
    daemon_request_loop_thread.daemon = True
    daemon_request_loop_thread.start() 
  
    logger.info("******** before client connection ******************")
    client_connected = False
    timeout = 40
    while not client_connected and timeout > 0:
      client_connected = connection_checker.isConnected()
      timeout = timeout - 1
      time.sleep(1)
      
    logger.info("******** first mode: client connection *************")
    while client_connected:
      client_connected = connection_checker.isConnected()
      time.sleep(1)
      
    logger.info("******** client disconnection **********************")
    daemon.shutdown(disconnect=True) #stop the request loop
    daemon.sock.close() # free the port
    
    del(daemon) 
    
    logger.info("******** second mode: waiting for jobs to finish****")
    jobs_running = True
    while jobs_running:
      jobs_running = not workflow_engine.engine_loop.are_jobs_and_workflow_done()
      time.sleep(1)
    
    logger.info("******** jobs are done ! ***************************")
    workflow_engine.engine_loop_thread.stop()

    sch.clean()
    sys.exit()
  
  if not len(sys.argv) == 3 and not len(sys.argv) == 4:
    sys.stdout.write("start_workflow_engine takes 2 arguments:\n")
    sys.stdout.write("   1. resource id \n")
    sys.stdout.write("   2. name of the engine object. \n")
  else:  
    resource_id = sys.argv[1]
    engine_name = sys.argv[2]
    if len(sys.argv) == 3:
      main(resource_id, engine_name)
    if len(sys.argv) == 4:
      main(resource_id, engine_name, sys.argv[3])



#! /usr/bin/env python

'''
@author: Jinpeng LI
@contact: mr.li.jinpeng@gmail.com

@organization: I2BM, Neurospin, Gif-sur-Yvette, France
@organization: CATI, France
@organization: U{IFR 49<http://www.ifr49.org>}

@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''

'''
start to check the requirement on the server side
'''
import os
import sys
import socket
import subprocess
import platform

path2somawf = os.path.dirname(os.path.realpath(__file__))
path2somawfpy = os.path.join(path2somawf,"..","..")
path2somawf_setup_server = os.path.realpath(__file__)
sys.path.append(path2somawfpy)


def SetPathToEnvVar(KeyPath,NewPath):
    os.environ[KeyPath] = NewPath
    res = ""
    if platform.system()=='Windows':
        res = "set %s=%s"%(KeyPath,NewPath)
    else:
        res = "export %s=%s"%(KeyPath,NewPath)
    return res
        
def AddPathToEnvVar(KeyPath,NewPath):
    os.environ[KeyPath] = "%s%s%s"%(NewPath,os.pathsep,os.environ.get(KeyPath))
    res = ""
    
    if platform.system()=='Windows':
        res = "set %s=%s%s%%%s%%"%(KeyPath,NewPath,os.pathsep,KeyPath)
    else:
        res = "export %s=%s%s${%s}"%(KeyPath,NewPath,os.pathsep,KeyPath)
        
    return res
        


def SetupServerEnvVar(path2somawfpy):
    '''
    Configurate the environment variable configuration
    '''
    envlines2add = [ ]
    
    envlines2add.append(AddPathToEnvVar("PYTHONPATH",path2somawfpy))

    import socket
    if socket.gethostname()=="gabriel.intra.cea.fr":
        envlines2add.append(AddPathToEnvVar("PYTHONPATH","/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/lib/python2.7"))
        envlines2add.append(AddPathToEnvVar("PYTHONPATH","/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/lib/python2.7/site-packages"))
        envlines2add.append(AddPathToEnvVar("PATH","/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/bin"))
        envlines2add.append(AddPathToEnvVar("LD_LIBRARY_PATH","/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/lib"))
        envlines2add.append(AddPathToEnvVar("LD_LIBRARY_PATH","/i2bm/brainvisa/CentOS-5.3-x86_64/pbs_drmaa-1.0.13/lib"))
        envlines2add.append(AddPathToEnvVar("LD_LIBRARY_PATH","/usr/lib64/openmpi/lib"))
        envlines2add.append(SetPathToEnvVar("DRMAA_LIBRARY_PATH","/i2bm/brainvisa/CentOS-5.3-x86_64/pbs_drmaa-1.0.13/lib/libdrmaa.so"))
    
    return envlines2add


envlines2add=SetupServerEnvVar(path2somawfpy)

'''
Start to check the requirement on the server side
'''
req_version = (2,7)
cur_version = sys.version_info

if (cur_version < req_version 
and socket.gethostname()=="gabriel.intra.cea.fr" 
and os.path.exists("/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/bin")):
    '''
    The default python in gabriel is too old to soma-workflow
    Use new version of python to setup since we have defined some enviroment in SetupServerEnvVar for gabriel.intra.cea.fr
    '''
    print "Use nested python to install"

    strcmd="/i2bm/brainvisa/CentOS-5.3-x86_64/python-2.7.3/bin/python '%s'"%(path2somawf_setup_server)

    i=1
    while i < len(sys.argv):
      # print sys.argv[i]
      strcmd=strcmd+" %s"%(sys.argv[i])
      i=i+1

    #print strcmd

    os.system(strcmd)
    sys.exit(0)
    

if cur_version < req_version or cur_version >= (3,0):
    print "This program requires a python version >= 2.7 and please update your python %s \
    to latest version from python2."%(repr(cur_version))
    sys.exit(0)
'''
end to check the requirement on the server side
'''


from soma_workflow.configuration import AddLineDefintions2BashrcFile,WriteOutConfiguration
import soma_workflow.configuration as configuration


def GetQueueNamesOnPBSTORQUEServer():
    import re
    
    info_queue =[]
    
    info_queue_out=subprocess.check_output(['qstat', '-Q'])
    
    info_queue_lines=info_queue_out.split('\n')
    
    sline_idx=0
    for info_queue_line in info_queue_lines:
        if sline_idx >= 2: # skip the first line since it is the header
            sline = info_queue_line.strip()
            ssline = sline.split()
            if len(ssline)>=1 :
                if re.match("^[a-zA-Z]", ssline[0]):
                    #print repr(ssline)
                    info_queue.append(ssline[0])
        sline_idx+=1
    
    return info_queue

def SetupConfigurationFileOnServer(userid,  ip_address_or_domain,   resource_id=None):
    """To setup the configuration file on the client part
     
    Args:
       userid (str):  user name on the server side
       ip_address_or_domain (str): the ip address or the domain of the server

    Raises:
       IOError, ValueError

    It will create the configuration at $HOME/.soma-workflow.cfg

    """
    #ouput the configuration file 
    import sys
    import os
    from ConfigParser import SafeConfigParser

    config_file_path = configuration.Configuration.search_config_path()
    
    if resource_id==None:
        resource_id="%s@%s"%(userid,ip_address_or_domain)
        
    
    home_dir = configuration.Configuration.get_home_dir() 
    install_prefix=os.path.join(home_dir,".soma-workflow")
    
    if config_file_path==None:
        config_file_path=os.path.join(home_dir,".soma-workflow.cfg")
    
#    print "config_file_path="+config_file_path
#    print "resource_id="+resource_id
#    print "home_dir="+home_dir
#    print "config_path="+config_path
#    print "install_prefix="+install_prefix
    
    config_parser = SafeConfigParser()
    config_parser.add_section(resource_id)
    
    config_parser.set(resource_id,configuration.CFG_DATABASE_FILE,          os.path.join(install_prefix,"soma_workflow.db"))
    config_parser.set(resource_id,configuration.CFG_TRANSFERED_FILES_DIR,   os.path.join(install_prefix,"transfered-files"))
    config_parser.set(resource_id,configuration.CFG_NAME_SERVER_HOST,       ip_address_or_domain)
    config_parser.set(resource_id,configuration.CFG_SERVER_NAME,            "soma_workflow_database_"+userid)
    
    config_parser.set(resource_id,configuration.OCFG_SERVER_LOG_FILE,       os.path.join(install_prefix,"logs","log_server"))
    config_parser.set(resource_id,configuration.OCFG_SERVER_LOG_FORMAT,     "%(asctime)s => line %(lineno)s: %(message)s")
    config_parser.set(resource_id,configuration.OCFG_SERVER_LOG_LEVEL,      "ERROR")
    config_parser.set(resource_id,configuration.OCFG_ENGINE_LOG_DIR,        os.path.join(install_prefix,"logs"))
    config_parser.set(resource_id,configuration.OCFG_ENGINE_LOG_FORMAT,     "%(asctime)s => %(module)s line %(lineno)s: %(message)s                 %(threadName)s")
    config_parser.set(resource_id,configuration.OCFG_ENGINE_LOG_LEVEL,      "ERROR")

    info_queue=GetQueueNamesOnPBSTORQUEServer()
    
    str_info_q="{15} "
    
    for e_info_queue in info_queue:
        str_info_q=str_info_q+e_info_queue+"{15} "
    str_info_q=str_info_q.strip()
    
    config_parser.set(resource_id,configuration.OCFG_MAX_JOB_IN_QUEUE,      str_info_q)
    
    WriteOutConfiguration(config_parser,config_file_path)


AddLineDefintions2BashrcFile(envlines2add)


import argparse
parser = argparse.ArgumentParser()
parser.add_argument("-r", help="resource id")

args = parser.parse_args()
if not args.r or args.r == '':
    print "Please enter resource id with -r resource_id"
    sys.exit(0)

resource_id=args.r


import getpass
userid=getpass.getuser()
ip_address_or_domain=socket.gethostname()
#resource_id="%s@%s"%(userid,ip_address_or_domain)

SetupConfigurationFileOnServer(userid,ip_address_or_domain,resource_id)

 
lines2cmd = [
             "kill $(ps -ef | grep 'python -m soma_workflow.start_database_server' | grep '%s' \
| grep -v grep | awk '{print $2}')"%(userid),
            "rm ~/.soma-workflow/*.db",
            "rm -rf ~/.soma-workflow/transfered-files/*",
            "rm -rf ~/.soma-workflow/logs/*",
            "mkdir ~/.soma-workflow/transfered-files",
            "mkdir ~/.soma-workflow/logs",
            "python -m soma_workflow.start_database_server %s & bg"%(resource_id)
             ]


for line2cmd in lines2cmd:
    os.system("echo '%s' "%(line2cmd))
    os.system(line2cmd)




import os
import sys

from soma_workflow.test import test_workflow
from soma_workflow.client import Helper

if __name__ == '__main__':

  directory = "/tmp/"
  
  simple_wf_examples = test_workflow.WorkflowExamples(
                                             with_transfers=False,
                                             with_shared_resource_path=False)
  tr_wf_examples = test_workflow.WorkflowExamples(
                                             with_transfers=True,
                                             with_shared_resource_path=False)
  srp_wf_examples = test_workflow.WorkflowExamples(
                                             with_transfers=False,
                                             with_shared_resource_path=True)
  workflows = []
  workflows.append(("multiple", simple_wf_examples.multiple_simple_example()))
  workflows.append(("special_command", simple_wf_examples.special_command()))
  workflows.append(("mutiple_transfer", tr_wf_examples.multiple_simple_example()))
  workflows.append(("special_command_transfer", tr_wf_examples.special_command()))
  workflows.append(("special_transfer", tr_wf_examples.special_transfer()))
  workflows.append(("mutiple_crp", srp_wf_examples.multiple_simple_example()))
  workflows.append(("special_command_crp", srp_wf_examples.special_command()))
    
  ret_value = 0

  for workflow_name, workflow in workflows:
    print "--------------------------------------------------------------"
    print workflow_name

    file_path = os.path.join(directory, "json_" + workflow_name + ".wf")
    Helper.serialize(file_path, workflow)      

    new_workflow = Helper.unserialize(file_path)

    if not new_workflow.attributs_equal(workflow):
      print "FAILED !!"
      ret_value = 1
    else:
      print "OK"
    
    try:
      os.remove(file_path)
    except IOError:
      pass  

  if ret_value == 0:
    print "\nAll test ran with success."
  else:
    print "\nOne or several tests failed."

  sys.exit(ret_value)
